@@ -1,6 +1,6 @@
-#! /usr/bin/env bash 
+#! /usr/bin/env bash
 #
-# Copyright (c) 2004-2006 The Trustees of Indiana University and Indiana
+# Copyright (c) 2004-2009 The Trustees of Indiana University and Indiana
 #                         University Research and Technology
 #                         Corporation.  All rights reserved.
 # Copyright (c) 2004-2005 The University of Tennessee and The University
@@ -11,8 +11,6 @@
 # Copyright (c) 2004-2005 The Regents of the University of California.
 #                         All rights reserved.
 # Copyright (c) 2007-2010 Cisco Systems, Inc.  All rights reserved.
-# Copyright (c) 2010      Oracle and/or its affilitates.  All rights 
-#                         reserved.
 # $COPYRIGHT$
 # 
 # Additional copyrights may follow
@@ -29,6 +27,14 @@
 unalias test sort cut xargs echo 2> /dev/null
 unset GREP_COLORS
 
+# Solaris patch is not compatible with Linux patch
+uname_s=`uname -s`
+if test "$uname_s" = "SunOS" ; then
+    patch="gpatch"
+else
+    patch="patch"
+fi
+
 ##############################################################################
 #
 # User-definable parameters (search path and minimum supported versions)
@@ -89,6 +95,13 @@ mca_no_config_list_file="mca_no_config_list"
 mca_no_config_env_file="mca_no_config_env"
 mca_m4_include_file="config/mca_m4_config_include.m4"
 mca_m4_config_env_file="mca_m4_config_env"
+
+ext_no_configure_components_file="config/ext_no_configure_components.m4"
+ext_no_config_list_file="ext_no_config_list"
+ext_no_config_env_file="ext_no_config_env"
+ext_m4_include_file="config/ext_m4_config_include.m4"
+ext_m4_config_env_file="ext_m4_config_env"
+
 autogen_subdir_file="autogen.subdirs"
 topdir_file="opal/include/opal_config_bottom.h"
 
@@ -179,6 +192,7 @@ find_app() {
     local version="0.0.0"
     local min_version="99.99.99"
     local found=0
+    local versions_found=""
     local tmpIFS=$IFS
 
     eval "min_version=\"\$ompi_${app_name}_version\""
@@ -201,6 +215,7 @@ find_app() {
             found=1
             break
         fi
+        versions_found="${versions_found} ${version}"
     done
 
     IFS="$tmpIFS"
@@ -208,6 +223,7 @@ find_app() {
     if test "$found" = "0" ; then
 	cat <<EOF
 I could not find a recent enough copy of ${app_name}.
+Need at least $min_version but found $versions_found in $search_path.
 I am gonna abort.  :-(
 
 Please make sure you are using at least the following versions of the
@@ -450,7 +466,7 @@ EOF
        # enough Libtool that dosn't need this patch.  But don't alarm the
        # user and make them think that autogen failed if this patch fails --
        # make the errors be silent.
-       patch -N -p0 < config/ltmain_pgi_tp.diff >/dev/null 2>&1
+       $patch -N -p0 < config/ltmain_pgi_tp.diff >/dev/null 2>&1
        if test ! $? -eq 0 ; then
            echo "      -- your libtool doesn't need this! yay!"
        fi
@@ -473,6 +489,8 @@ EOF
    {
 EOF
 #'
+            rm -f ltdl.c.orig
+            rm -f ltdl.c.rej
         else
             echo "      -- your libtool doesn't need this! yay!"
         fi
@@ -482,6 +500,7 @@ EOF
               -z "`grep otool64 config/ltmain.sh`"; then
             patch -N -p0 < config/ltmain_otool.diff
             rm -f config/ltmain.sh.orig
+            rm -f config/ltmain.sh.rej
         else
             echo "      -- your libtool doesn't need this! yay!"
         fi
@@ -490,6 +509,7 @@ EOF
         if test -r opal/libltdl/loaders/dlopen.c && \
             test ! -z "`grep 'filename, LT_LAZY_OR_NOW' opal/libltdl/loaders/dlopen.c`"; then
             patch -N -p0 < config/libltdl_dlopen_global.diff
+            rm -f opal/libltdl/loaders/dlopen.c.rej
         else
             echo "      -- your libltdl doesn't need this! yay!"
         fi
@@ -500,6 +520,11 @@ EOF
             mv configure.new configure
             chmod a+x configure
         fi
+        echo "   ++ preopen error masking ib libltdl"
+        if test -r opal/libltdl/loaders/preopen.c; then
+            patch -N -p0 < config/libltdl-preopen-error.diff
+            rm -f opal/libltdl/loaders/preopen.c.rej
+        fi
 
         # See
         # http://lists.gnu.org/archive/html/bug-libtool/2008-05/msg00045.html.
@@ -535,7 +560,14 @@ EOF
     fi
 
     if test -f $topdir_file; then 
-        run_and_check $ompi_aclocal -I config
+        flags=
+        if test "$want_orte" = "1"; then
+            flags="$flags -I orte/config"
+        fi
+        if test "$want_ompi" = "1"; then
+            flags="$flags -I ompi/config"
+        fi
+        run_and_check $ompi_aclocal -I config -I opal/config $flags
     else
         run_and_check $ompi_aclocal
     fi
@@ -623,6 +655,8 @@ EOF
         #   http://www.open-mpi.org/community/lists/devel/2008/11/4920.php
         echo "   ++ patching for Sun Studio Fortran compilers"
         patch -N -p0 < config/lt-sun-fortran.diff > /dev/null 2>&1
+        rm -f libtool.m4.orig
+        rm -f libtool.m4.rej
         # We must touch aclocal.m4 here, because it must be newer than
         # libtool.m4, otherwise a whole bunch of Automake-mandated
         # timestamps may be off (depending on the resolution of
@@ -691,8 +725,13 @@ run_no_configure_component() {
     noconf_component="$5"
 
     # Write out to two files (they're merged at the end)
-    noconf_list_file="$noconf_ompi_topdir/$mca_no_config_list_file"
-    noconf_env_file="$noconf_ompi_topdir/$mca_no_config_env_file"
+    if test $6 = "mca" ; then
+        noconf_list_file="$noconf_ompi_topdir/$mca_no_config_list_file"
+        noconf_env_file="$noconf_ompi_topdir/$mca_no_config_env_file"
+    else
+        noconf_list_file="$noconf_ompi_topdir/$ext_no_config_list_file"
+        noconf_env_file="$noconf_ompi_topdir/$ext_no_config_env_file"
+    fi
 
     cat >> "$noconf_list_file" <<EOF
 dnl ----------------------------------------------------------------
@@ -741,8 +780,13 @@ run_m4_configure_component() {
     m4conf_component="$5"
 
     # Write out to two files (they're merged at the end)
-    m4conf_list_file="$m4conf_ompi_topdir/$mca_no_config_list_file"
-    m4conf_env_file="$m4conf_ompi_topdir/$mca_m4_config_env_file"
+    if test $6 = "mca" ; then
+        m4conf_list_file="$m4conf_ompi_topdir/$mca_no_config_list_file"
+        m4conf_env_file="$m4conf_ompi_topdir/$mca_m4_config_env_file"
+    else
+        m4conf_list_file="$m4conf_ompi_topdir/$ext_no_config_list_file"
+        m4conf_env_file="$m4conf_ompi_topdir/$ext_m4_config_env_file"
+    fi
 
     cat >> "$m4conf_list_file" <<EOF
 dnl ----------------------------------------------------------------
@@ -763,7 +807,16 @@ EOF
     # do.  By this point, there should already be a header and all
     # that.  m4_includes are relative to the currently included file,
     # so need the .. to get us from config/ to the topsrcdir again.
-    echo "m4_include(${m4conf_project}/mca/${m4conf_framework}/${m4conf_component}/configure.m4)" >> "$m4conf_ompi_topdir/$mca_m4_include_file"
+    # Note that the ompi contrib system comes through here as well,
+    # but they directly m4 include their own configure.m4 files, so we
+    # skip them here.
+    if test "$m4conf_framework" = "contrib" ; then
+        true
+    elif test $6 = "mca" ; then
+        echo "m4_include(${m4conf_project}/mca/${m4conf_framework}/${m4conf_component}/configure.m4)" >> "$m4conf_ompi_topdir/$mca_m4_include_file"
+    else
+        echo "m4_include(${m4conf_project}/${m4conf_framework}/${m4conf_component}/configure.m4)" >> "$m4conf_ompi_topdir/$ext_m4_include_file"
+    fi
 
     cat <<EOF
 --> Adding to top-level configure m4-configure subdirs:
@@ -982,7 +1035,7 @@ EOF
                 rm -f "configure" "configure.ac*" "acinclude*" "aclocal.m4"
 
                 run_m4_configure_component "$pd_dir" "$pd_ompi_topdir" \
-                    "$pd_project" "$pd_framework" "$pd_component"
+                    "$pd_project" "$pd_framework" "$pd_component" "mca"
             fi
 
         elif test -f configure.ac -o -f configure.in; then
@@ -1021,7 +1074,198 @@ EOF
 EOF
             else
                 run_no_configure_component "$pd_dir" "$pd_ompi_topdir" \
-                    "$pd_project" "$pd_framework" "$pd_component"
+                    "$pd_project" "$pd_framework" "$pd_component" "mca"
+            fi
+        else
+	    cat <<EOF
+
+*** Nothing found; directory skipped
+***   `pwd`
+
+EOF
+        fi
+
+        # See if there's a file containing additional directories to
+        # traverse.  Shell scripts aren't too good at recursion (no
+        # local state!), so just have a child autogen.sh do the work.
+
+        if test -f $autogen_subdir_file; then
+            pd_subdir_start_dir="`pwd`"
+            echo ""
+            echo "==> Found $autogen_subdir_file -- sub-traversing..."
+            echo ""
+            for dir in `cat $autogen_subdir_file`; do
+                if test -d "$dir"; then
+                    echo "*** Running autogen.sh in $dir"
+                    echo "***   (started in $pd_subdir_start_dir)"
+                    cd "$dir"
+                    $pd_ompi_topdir/autogen.sh -l
+                    if test ! $? -eq 0 ; then
+                        echo "Error running autogen.sh -l in $dir.  Aborting."
+                        exit 1
+                    fi
+                    cd "$pd_subdir_start_dir"
+                    echo ""
+                fi
+            done
+            echo "<== Back in $pd_subdir_start_dir"
+            echo "<== autogen.sh continuing..."
+        fi
+
+        # Go back to the topdir
+
+        cd "$pd_cur_dir"
+    fi
+    unset PARAM_CONFIG_FILES PARAM_VERSION_FILE
+    unset pd_dir pd_ompi_topdir pd_cur_dir pd_component_type
+}
+
+process_ext_dir() {
+    pd_dir="$1"
+    pd_ompi_topdir="$2"
+    pd_project="$3"
+    pd_framework="$4"
+    pd_component="$5"
+
+    pd_cur_dir="`pwd`"
+
+    # Convert to absolutes
+
+    if test -d "$pd_dir"; then
+        cd "$pd_dir"
+        pd_abs_dir="`pwd`"
+        cd "$pd_cur_dir"
+    fi
+
+    if test -d "$pd_ompi_topdir"; then
+        cd "$pd_ompi_topdir"
+        pd_ompi_topdir="`pwd`"
+        cd "$pd_cur_dir"
+    fi
+
+    # clean our environment a bit, since we might evaluate a configure.params
+    unset PARAM_CONFIG_FILES
+    PARAM_CONFIG_PRIORITY="0"
+
+    if test -d "$pd_dir"; then
+	cd "$pd_dir"
+
+	# See if the package doesn't want us to set it up
+
+	if test -f .ompi_no_gnu; then
+	    cat <<EOF
+
+*** Found .ompi_no_gnu file -- skipping GNU setup in:
+***   `pwd`
+
+EOF
+	elif test -f .ompi_ignore -a ! -f .ompi_unignore; then
+
+            # Note that if we have an empty (but existant)
+            # .ompi_unignore, then we ignore the .ompi_ignore file
+            # (and therefore build the component)
+
+	    cat <<EOF
+
+*** Found .ompi_ignore file -- skipping entire tree:
+***   `pwd`
+
+EOF
+
+        # Use && instead of -a here for the test conditions because if
+        # you use -a, then "test" will execute *all* condition clauses
+        # (even if the first one is false), meaning that grep will
+        # fail if there is no .ompi_unignore file.  If you use &&,
+        # then the latter tests will not be executed if a prior one
+        # fails (i.e., grep won't run if .ompi_unignore does not
+        # exist).
+
+        elif test -f .ompi_ignore && \
+             test -s .ompi_unignore && \
+             test -z "`$egrep $USER\$\|$USER@$HOST .ompi_unignore`" ; then
+
+            # If we have a non-empty .ompi_unignore and our username
+            # is in there somewhere, we ignore the .ompi_ignore (and
+            # therefore build the component).  Otherwise, this
+            # condition is true and we don't configure.
+
+	    cat <<EOF
+
+*** Found .ompi_ignore file and .ompi_unignore didn't invalidate -- 
+*** skipping entire tree:
+***   `pwd`
+
+EOF
+	elif test "$pd_abs_dir" != "$pd_ompi_topdir" -a -x autogen.sh; then
+	    cat <<EOF
+
+*** Found custom autogen.sh file in:
+***   `pwd`
+
+EOF
+	    ./autogen.sh
+            if test ! $? -eq 0 ; then
+                echo "Error running autogen.sh -l in `pwd`.  Aborting."
+                exit 1
+            fi
+        elif test -f configure.params -a -f configure.m4 ; then
+            cat <<EOF
+
+*** Found configure.params and configure.m4
+***   `pwd`
+
+EOF
+            . ./configure.params
+            if test -z "$PARAM_CONFIG_FILES"; then
+                cat <<EOF
+*** No PARAM_CONFIG_FILES!
+*** Nothing to do -- skipping this directory
+EOF
+            else
+                # temporary workaround - remove possibly there configure code
+                rm -f "configure" "configure.ac*" "acinclude*" "aclocal.m4"
+
+                run_m4_configure_component "$pd_dir" "$pd_ompi_topdir" \
+                    "$pd_project" "$pd_framework" "$pd_component" "ext"
+            fi
+
+        elif test -f configure.ac -o -f configure.in; then
+            # If we have configure.ac or configure.in, run the GNU
+            # tools here
+
+	    cat <<EOF
+
+*** Found configure.(in|ac)
+***   `pwd`
+
+EOF
+            run_gnu_tools "$pd_ompi_topdir"
+
+        elif test -f configure.params -a -f configure.stub; then
+	    cat <<EOF
+
+*** Found configure.params and configure.stub
+***   `pwd`
+
+EOF
+            run_gnu_tools "$pd_ompi_topdir"
+
+        elif test -f configure.params; then
+	    cat <<EOF
+
+*** Found configure.params
+***   `pwd`
+
+EOF
+            . ./configure.params
+            if test -z "$PARAM_CONFIG_FILES"; then
+                cat <<EOF
+*** No PARAM_CONFIG_FILES!
+*** Nothing to do -- skipping this directory
+EOF
+            else
+                run_no_configure_component "$pd_dir" "$pd_ompi_topdir" \
+                    "$pd_project" "$pd_framework" "$pd_component" "ext"
             fi
         else
 	    cat <<EOF
@@ -1141,6 +1385,76 @@ process_framework() {
     fi
 }
 
+process_ext_framework() {
+    framework_path="$1"
+    rg_cwd="$2"
+    project="$3"
+    framework="$4"
+
+    if test -d "$framework_path" ; then
+        framework_ext_list="$framework_ext_list $framework"
+
+        # Add the framework's configure file into configure,
+        # if there is one
+        if test -r "${framework_path}/configure.m4" ; then
+            echo "m4_include(${framework_path}/configure.m4)" >> "$ext_m4_include_file"
+        fi
+        echo "AC_CONFIG_FILES(${framework_path}/Makefile)" >> "$ext_no_config_list_file"
+
+        rm -f "$ext_no_config_env_file" "$ext_m4_config_env_file"
+        touch "$ext_no_config_env_file" "$ext_m4_config_env_file"
+
+        for component_path in "$framework_path"/*; do
+            if test -d "$component_path"; then
+                if test -f "$component_path/configure.in" -o \
+                    -f "$component_path/configure.params" -o \
+                    -f "$component_path/configure.ac"; then
+
+                    component=`basename "$component_path"`
+
+                    process_ext_dir "$component_path" "$rg_cwd" \
+                        "$project" "$framework" "$component"
+                fi
+            fi
+        done
+
+        # make list of components that are "no configure".
+        # Sort the list by priority (stable, so things stay in
+        # alphabetical order at the same priority), then munge
+        # it into form we like
+        component_list=
+        component_list_sort $ext_no_config_env_file
+        component_list_define="m4_define([ext_${framework}_no_config_component_list], ["
+        component_list_define_first="1"
+        for component in $component_list ; do
+            if test "$component_list_define_first" = "1"; then
+                component_list_define="${component_list_define}${component}"
+                component_list_define_first="0"
+            else
+                component_list_define="${component_list_define}, ${component}"
+            fi
+        done
+        component_list_define="${component_list_define}])"
+        echo "$component_list_define" >> "$ext_no_configure_components_file"
+
+        # make list of components that are "m4 configure"
+        component_list=
+        component_list_sort $ext_m4_config_env_file
+        component_list_define="m4_define([ext_${framework}_m4_config_component_list], ["
+        component_list_define_first="1"
+        for component in $component_list ; do
+            if test "$component_list_define_first" = "1"; then
+                component_list_define="${component_list_define}${component}"
+                component_list_define_first="0"
+            else
+                component_list_define="${component_list_define}, ${component}"
+            fi
+        done
+        component_list_define="${component_list_define}])"
+        echo "$component_list_define" >> "$ext_no_configure_components_file"
+    fi
+}
+
 process_project() {
     project_path="$1"
     rg_cwd="$2"
@@ -1170,9 +1484,30 @@ process_project() {
     # process contributed software packages (ompi/contrib/*)
     if test "$project" = "ompi"; then
         for contrib_path in $project_path/contrib/*; do
-            process_dir $contrib_path $rg_cwd
+            process_dir $contrib_path $rg_cwd ompi contrib `basename $contrib_path`
         done
     fi
+
+    # Extensions interface only applicable to ompi project
+    if test $project = "ompi" ; then
+        # Process interface extensions
+        framework_ext_list=""
+        process_ext_framework $project_path/mpiext $rg_cwd $project "mpiext"
+
+        # make list of frameworks for this project
+        framework_ext_list_define="m4_define([ext_${project}_framework_list], ["
+        framework_ext_list_define_first="1"
+        for framework in $framework_ext_list ; do
+            if test "$framework_ext_list_define_first" = "1"; then
+                framework_ext_list_define="${framework_ext_list_define}${framework}"
+                framework_ext_list_define_first="0"
+            else
+                framework_ext_list_define="${framework_ext_list_define}, ${framework}"
+            fi
+        done
+        framework_ext_list_define="${framework_ext_list_define}])"
+        echo "$framework_ext_list_define" >> "$ext_no_configure_components_file"
+    fi
 }
 
 
@@ -1217,6 +1552,34 @@ dnl
 dnl This file is automatically created by autogen.sh; it should not
 dnl be edited by hand!!
 
+EOF
+
+    # [Re-]Create the ext_component_list file
+    rm -f "$ext_no_configure_components_file" "$ext_no_config_list_file" \
+        "$ext_no_config_env_file" "$ext_m4_config_env_file" "$ext_m4_include_file"
+    touch "$ext_no_configure_components_file" "$ext_no_config_list_file" \
+        "$ext_m4_config_env_file" "$ext_m4_include_file"
+
+    # create header for the component m4 include file
+    cat > "$ext_m4_include_file" <<EOF
+dnl
+dnl \$HEADER
+dnl
+
+dnl This file is automatically created by autogen.sh; it should not
+dnl be edited by hand!!
+
+EOF
+
+    #create header for the component config file
+    cat > "$ext_no_configure_components_file" <<EOF
+dnl
+dnl \$HEADER
+dnl
+
+dnl This file is automatically created by autogen.sh; it should not
+dnl be edited by hand!!
+
 EOF
 
     # Now run the config in every directory in <location>/mca/*/*
@@ -1254,6 +1617,11 @@ EOF
     project_list_define="${project_list_define}])"
     echo "$project_list_define" >> "$mca_no_configure_components_file"
 
+    # create the m4 defines for the list of projects where MPI Extensions
+    # apply, which is only 'ompi'
+    project_list_define="m4_define([ext_project_list], [ompi])"
+    echo "$project_list_define" >> "$ext_no_configure_components_file"
+
 
     cat >> "$mca_no_configure_components_file" <<EOF
 
@@ -1264,10 +1632,22 @@ AC_DEFUN([MCA_NO_CONFIG_CONFIG_FILES],[
 
 `cat $mca_no_config_list_file`
 ])dnl
+EOF
+
+    cat >> "$ext_no_configure_components_file" <<EOF
+
+dnl List all the no-configure components that we found, and AC_DEFINE
+dnl their versions
+
+AC_DEFUN([EXT_NO_CONFIG_CONFIG_FILES],[
+
+`cat $ext_no_config_list_file`
+])dnl
 EOF
 
     # Remove temp files
     rm -f $mca_no_config_list_file $mca_no_config_env_file $mca_m4_config_env_file
+    rm -f $ext_no_config_list_file $ext_no_config_env_file $ext_m4_config_env_file
 
     # Finally, after we found all the no-configure MCA components, run
     # the config in the top-level directory
@@ -1318,14 +1698,14 @@ echo "[Checking] command line parameters"
 
 want_local=no
 ompidir=
-no_ompi=0
-no_orte=0
+want_ompi=1
+want_orte=1
 for arg in $*; do
     case $arg in
     -l) want_local=yes ;;
     -ompidir|--ompidir|-ompi|--ompi) ompidir="$2" ;;
-    -no-ompi) no_ompi=1 ;;
-    -no-orte) no_orte=1 ;;
+    -no-ompi|--no-ompi) want_ompi=0 ;;
+    -no-orte|--no-orte) want_orte=0 ;;
     *) ;;
     esac
     shift
@@ -1351,13 +1731,36 @@ fi
 # top-level directory, or somewhere else.
 if test -f VERSION -a -f configure.ac -a -f $topdir_file ; then
     # locations to look for mca modules
-    config_project_list="opal orte ompi"
-    if test "$no_ompi" = "1" ; then
-        config_project_list="opal orte"
+    config_project_list="opal"
+    project_file=config/project_list.m4
+    rm -f $project_file
+    cat > $project_file <<EOF
+dnl This file is automatically generated by autogen.sh
+dnl Manual changes will be lost!
+
+dnl Listing of sub-layers to include
+m4_define([project_opal], [1])
+EOF
+    project_name_long="Open Portability Access Layer"
+    project_name_short="open-pal"
+    if test "$want_orte" = "1" ; then
+        config_project_list="$config_project_list orte"
+        echo "m4_define([project_orte], [1])" >> $project_file
+        project_name_long="Open MPI Run Time Environment"
+        project_name_short="openmpi-rte"
     fi
-    if test "$no_orte" = "1" ; then
-        config_project_list="opal"
+    if test "$want_ompi" = "1" ; then
+        config_project_list="$config_project_list ompi"
+        echo "m4_define([project_ompi], [1])" >> $project_file
+        project_name_long="Open MPI"
+        project_name_short="openmpi"
     fi
+    cat >> $project_file <<EOF
+
+dnl Project name
+m4_define([project_name_long], [$project_name_long])
+m4_define([project_name_short], [$project_name_short])
+EOF
     echo "Configuring projects: $config_project_list"
 
     # Top level of OMPI tree
