@@ -1,9 +1,11 @@
 /*
- This is part of the OTF library. Copyright by ZIH, TU Dresden 2005-2008.
+ This is part of the OTF library. Copyright by ZIH, TU Dresden 2005-2010.
  Authors: Andreas Knuepfer, Holger Brunst, Ronny Brendel, Thomas Kriebitzsch
 */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
 
 
 #include <stddef.h>
@@ -50,6 +52,7 @@ static const char* Helptext[] = {
 "      -z <zlevel>   write compressed output                       \n",
 "                    zlevel reaches from 0 to 9 where 0 is no      \n",
 "                    compression and 9 is the highest level        \n",
+"      -l            write long OTF format                         \n",
 "      -p            show progress                                 \n",
 "                                                                  \n",
 "                                                                  \n", NULL };
@@ -71,6 +74,7 @@ int main ( int argc, char** argv ) {
 	int read_snaps= 0;
 	int nstreams = 1;
 	int nfiles = 200;
+	int longformat = 0;
 	int showprogress= 0;
 	int i;
 
@@ -94,6 +98,7 @@ int main ( int argc, char** argv ) {
 	uint32_t progress_counter= 0;
 
 	uint64_t retde;
+	uint64_t retma;
 	uint64_t retev;
 	uint64_t retst;
 	uint64_t retsn;
@@ -153,6 +158,10 @@ int main ( int argc, char** argv ) {
 
 			++i;
 
+		} else if ( 0 == strcmp( "-l", argv[i] ) ) {
+
+			longformat = 1;
+
 		} else if ( 0 == strcmp( "-p", argv[i] ) ) {
 
 			showprogress= 1;
@@ -165,7 +174,7 @@ int main ( int argc, char** argv ) {
 
 		} else if ( 0 == strcmp( "-V", argv[i] ) ) {
 		
-			printf( "%u.%u.%u \"%s\"\n", OTF_VERSION_MAYOR, OTF_VERSION_MINOR,
+			printf( "%u.%u.%u \"%s\"\n", OTF_VERSION_MAJOR, OTF_VERSION_MINOR,
 				OTF_VERSION_SUB, OTF_VERSION_STRING);
 			exit( 0 );
 
@@ -231,7 +240,11 @@ int main ( int argc, char** argv ) {
 	fcb.writer = OTF_Writer_open( outfile, nstreams, manager );
 	OTF_Writer_setBufferSizes( fcb.writer, writerbuffersize );
 	OTF_Writer_setCompression( fcb.writer, compression );
-	
+	if( longformat )
+		OTF_Writer_setFormat( fcb.writer, OTF_WSTREAM_FORMAT_LONG );
+	else
+                OTF_Writer_setFormat( fcb.writer, OTF_WSTREAM_FORMAT_SHORT );
+
 	mc = OTF_Reader_getMasterControl( reader );
 
 	/* set your own handler functions */
@@ -352,6 +365,18 @@ int main ( int argc, char** argv ) {
 	OTF_HandlerArray_setFirstHandlerArg( handlers, 
         &fcb, OTF_COLLOP_RECORD );
 
+	OTF_HandlerArray_setHandler( handlers, 
+		(OTF_FunctionPointer*) handleBeginCollectiveOperation,
+		OTF_BEGINCOLLOP_RECORD );
+	OTF_HandlerArray_setFirstHandlerArg( handlers, 
+        &fcb, OTF_BEGINCOLLOP_RECORD );
+
+	OTF_HandlerArray_setHandler( handlers, 
+		(OTF_FunctionPointer*) handleEndCollectiveOperation,
+		OTF_ENDCOLLOP_RECORD );
+	OTF_HandlerArray_setFirstHandlerArg( handlers, 
+        &fcb, OTF_ENDCOLLOP_RECORD );
+
 	OTF_HandlerArray_setHandler( handlers, 
 		(OTF_FunctionPointer*) handleRecvMsg,
 		OTF_RECEIVE_RECORD );
@@ -387,7 +412,43 @@ int main ( int argc, char** argv ) {
 		OTF_FILEOPERATION_RECORD );
 	OTF_HandlerArray_setFirstHandlerArg( handlers, &fcb, 
 		OTF_FILEOPERATION_RECORD );
-			
+
+	OTF_HandlerArray_setHandler( handlers, 
+		(OTF_FunctionPointer*) handleBeginFileOperation,
+		OTF_BEGINFILEOP_RECORD );
+	OTF_HandlerArray_setFirstHandlerArg( handlers, &fcb, 
+		OTF_BEGINFILEOP_RECORD );
+
+	OTF_HandlerArray_setHandler( handlers, 
+		(OTF_FunctionPointer*) handleEndFileOperation,
+		OTF_ENDFILEOP_RECORD );
+	OTF_HandlerArray_setFirstHandlerArg( handlers, &fcb, 
+		OTF_ENDFILEOP_RECORD );
+
+        OTF_HandlerArray_setHandler( handlers, 
+                (OTF_FunctionPointer*) handleRMAPut,
+                OTF_RMAPUT_RECORD );
+        OTF_HandlerArray_setFirstHandlerArg( handlers, &fcb,
+                OTF_RMAPUT_RECORD );
+
+        OTF_HandlerArray_setHandler( handlers, 
+                (OTF_FunctionPointer*) handleRMAPutRemoteEnd,
+                OTF_RMAPUTRE_RECORD );
+        OTF_HandlerArray_setFirstHandlerArg( handlers, &fcb,
+                OTF_RMAPUTRE_RECORD );
+
+        OTF_HandlerArray_setHandler( handlers, 
+                (OTF_FunctionPointer*) handleRMAGet,
+                OTF_RMAGET_RECORD );
+        OTF_HandlerArray_setFirstHandlerArg( handlers, &fcb,
+                OTF_RMAGET_RECORD );
+
+        OTF_HandlerArray_setHandler( handlers, 
+                (OTF_FunctionPointer*) handleRMAEnd,
+                OTF_RMAEND_RECORD );
+        OTF_HandlerArray_setFirstHandlerArg( handlers, &fcb,
+                OTF_RMAEND_RECORD );
+
 
 	/* snapshot records */
 
@@ -442,6 +503,12 @@ int main ( int argc, char** argv ) {
 	OTF_HandlerArray_setFirstHandlerArg( handlers, 
         &fcb, OTF_MESSAGESUMMARY_RECORD );
 
+	OTF_HandlerArray_setHandler( handlers, 
+		(OTF_FunctionPointer*) handleCollopSummary,
+		OTF_COLLOPSUMMARY_RECORD );
+	OTF_HandlerArray_setFirstHandlerArg( handlers, 
+        &fcb, OTF_COLLOPSUMMARY_RECORD );
+
 	OTF_HandlerArray_setHandler( handlers, 
 		(OTF_FunctionPointer*) handleFileOperationSummary,
 		OTF_FILEOPERATIONSUMMARY_RECORD );
@@ -452,7 +519,15 @@ int main ( int argc, char** argv ) {
 		(OTF_FunctionPointer*) handleFileGroupOperationSummary,
 		OTF_FILEGROUPOPERATIONSUMMARY_RECORD );
 	OTF_HandlerArray_setFirstHandlerArg( handlers, 
-        &fcb, OTF_FILEGROUPOPERATIONSUMMARY_RECORD );
+		&fcb, OTF_FILEGROUPOPERATIONSUMMARY_RECORD );
+
+	/* marker record types */
+
+	OTF_HandlerArray_setHandler( handlers, (OTF_FunctionPointer*) handleDefMarker, OTF_DEFMARKER_RECORD );
+	OTF_HandlerArray_setFirstHandlerArg( handlers, &fcb, OTF_DEFMARKER_RECORD );
+	
+	OTF_HandlerArray_setHandler( handlers, (OTF_FunctionPointer*) handleMarker, OTF_MARKER_RECORD );
+	OTF_HandlerArray_setFirstHandlerArg( handlers, &fcb, OTF_MARKER_RECORD );	
 
 	/* misc records */
 	
@@ -493,6 +568,14 @@ int main ( int argc, char** argv ) {
 		exit(1);
 	}
 
+	/* read markers */
+	retma= OTF_Reader_readMarkers( reader, handlers );
+	if( OTF_READ_ERROR == retma || 1 == fcb.error ) {
+		fprintf( stderr, "Error while reading marker records. aborting\n" );
+		FINISH_EVERYTHING;
+		exit(1);
+	}
+
 	
 	if ( 0 == showprogress ) {
 
