@@ -2,7 +2,7 @@
  * VampirTrace
  * http://www.tu-dresden.de/zih/vampirtrace
  *
- * Copyright (c) 2005-2008, ZIH, TU Dresden, Federal Republic of Germany
+ * Copyright (c) 2005-2010, ZIH, TU Dresden, Federal Republic of Germany
  *
  * Copyright (c) 1998-2005, Forschungszentrum Juelich, Juelich Supercomputing
  *                          Centre, Federal Republic of Germany
@@ -42,19 +42,18 @@
 
 #include <fstream>
 #include <assert.h>
-#include <stdlib.h>
 #include <string.h>
 
 using namespace std;
 
 
-#if (defined (VT_OMP))
+#if defined(HAVE_OMP) && HAVE_OMP
 #	include <omp.h>
-#else
+#else // HAVE_OMP
 #	define omp_get_num_threads() 1
 #	define omp_get_thread_num() 0
 #       define omp_get_max_threads() 1
-#endif
+#endif // HAVE_OMP
 
 
 #if SIZEOF_LONG == 4
@@ -253,7 +252,7 @@ int main( int argc, char** argv ) {
 	if( nfiles < (uint32_t)omp_get_max_threads() ) {
 		cerr << "The maximum number of open files is not allowed to be less than"
 			" the threadcount. Aborting" << endl;
-		abort();
+		exit(1);
 	}
 
 	switch(action) {
@@ -286,18 +285,25 @@ int main( int argc, char** argv ) {
 
 	if( err == true ) exit(1);
 
-	/* *** cut the ".otf"-suffix from input/output trace file name *** */
-	char* tmp;
+  /* cut the ".otf"-suffix from output/input trace file name */
+  if(!intrace.empty() || !outtrace.empty())
+  {
+    char* tmp = new char[((intrace.length() > outtrace.length()) ? 
+                          intrace.length() : outtrace.length()) + 1];
 
-	tmp = OTF_stripFilename(intrace.c_str());
-	intrace = tmp;
-	free(tmp);
-	
-	if(!outtrace.empty()) {
-		tmp = OTF_stripFilename(outtrace.c_str());
-		outtrace = tmp;
-		free(tmp);
-	}
+    if(!intrace.empty()) {
+      strncpy(tmp, intrace.c_str(), intrace.length());
+      tmp[intrace.length()] = '\0';
+      intrace = OTF_stripFilename(tmp);
+    }
+    if(!outtrace.empty()) {
+      strncpy(tmp, outtrace.c_str(), outtrace.length());
+      tmp[outtrace.length()] = '\0';
+      outtrace = OTF_stripFilename(tmp);
+    }
+  
+    delete[] tmp;
+  }
 
 	/* *** read the exclude symbols file, if there is one *** */
 	char* tfef = getenv("TRACEFILTER_EXCLUDEFILE");
@@ -333,10 +339,7 @@ int main( int argc, char** argv ) {
 		pmanager= OTF_FileManager_open( nfiles );
 		assert( pmanager );
 		preader = OTF_Reader_open( intrace.c_str(), pmanager );
-		if( !preader ) {
-			cerr << "Could not open input trace file \"" << intrace << "\". Aborting" << endl;
-			abort();
-		}
+		assert( preader );
 		phandlers = OTF_HandlerArray_open();
 		assert( phandlers );
 		OTF_MasterControl* pmc = OTF_Reader_getMasterControl( preader );
@@ -358,7 +361,7 @@ int main( int argc, char** argv ) {
 		uint64_t pretev= OTF_Reader_readEvents( preader, phandlers );
 		if( pretev == OTF_READ_ERROR ) {
 			cerr << "Error while reading Events. Aborting" << endl;
-			abort();
+			exit(1);
 		}
 
 		OTF_Reader_eventBytesProgress( preader, &pminbytestmp, &pcurbytestmp, &pmaxbytestmp );
@@ -372,7 +375,7 @@ int main( int argc, char** argv ) {
 		uint64_t pretde= OTF_Reader_readDefinitions( preader, phandlers );
 		if( pretde == OTF_READ_ERROR ) {
 			cerr << "Error while reading Definitions. Aborting" << endl;
-			abort();
+			exit(1);
 		}
 		
 		/* 3. insert all streams into the stream2filter mapping */
@@ -397,8 +400,8 @@ int main( int argc, char** argv ) {
 		OTF_MasterControl* mc = OTF_MasterControl_new(mcmanager);
 		assert(mc);
 		if( 0 == OTF_MasterControl_read( mc, intrace.c_str() ) ) {
-			cerr << "Could not read OTF Master Control File \"" << intrace << ".otf\". Aborting" << endl;
-			abort();
+			cerr << "Could not read Master Control File. Aborting" << endl;
+			exit(1);
 		}
 		
 		/* *** prepare the parallel construct *** */
@@ -408,12 +411,12 @@ int main( int argc, char** argv ) {
 		uint64_t lastprogressupdate = 0;
 		uint64_t progress_counter = 0;
 		bool erroroccured = false;
-		
-		#ifdef VT_OMP
-		#pragma omp parallel for firstprivate(mc,nfiles,minbytes,maxbytes) \
+
+#		if defined(HAVE_OMP) && HAVE_OMP		
+#		pragma omp parallel for firstprivate(mc,nfiles,minbytes,maxbytes) \
 			shared(fha,curbytes,readrecords,lastprogressupdate,progress_counter,erroroccured) \
 			private(retev,retst,retsn)
-		#endif
+#		endif // HAVE_OMP
 		for( int streamindex = 0; streamindex < maxstreams; ++streamindex )
 		{
 			/* *** init otf *** */
@@ -468,9 +471,9 @@ int main( int argc, char** argv ) {
 				while ( 0 != ( retev= OTF_RStream_readEvents( rstream, handlers ) ) ) {
 
 					if( OTF_READ_ERROR == retev ) {
-						#ifdef VT_OMP
-						#pragma omp critical (consoleout)
-						#endif
+#						if defined(HAVE_OMP) && HAVE_OMP
+#						pragma omp critical (consoleout)
+#						endif // HAVE_OMP
 						{
 							cerr << "Error while reading events. aborting" << endl;
 						}
@@ -483,9 +486,9 @@ int main( int argc, char** argv ) {
 						&minbytestmp, &curbytestmp, &maxbytestmp );
 
 					/* update the progress */
-					#ifdef VT_OMP
-					#pragma omp critical (progressupdate)
-					#endif
+#					if defined(HAVE_OMP) && HAVE_OMP
+#					pragma omp critical (progressupdate)
+#					endif // HAVE_OMP
 					{
 						curbytes += curbytestmp - lastbytestmp;
 
@@ -506,9 +509,9 @@ int main( int argc, char** argv ) {
 
 				retev = OTF_RStream_readEvents( rstream, handlers );
 				if( retev == OTF_READ_ERROR ) {
-					#ifdef VT_OMP
-					#pragma omp critical (consoleout)
-					#endif
+#					if defined(HAVE_OMP) && HAVE_OMP
+#					pragma omp critical (consoleout)
+#					endif // HAVE_OMP
 					{
 						cerr << "Error while reading events. aborting" << endl;
 					}
@@ -532,7 +535,7 @@ int main( int argc, char** argv ) {
 		}
 
 		if( erroroccured ) {
-			abort();
+			exit(1);
 		}
 
 		Filter filter;
@@ -604,7 +607,7 @@ int main( int argc, char** argv ) {
 			
 		} else {
 		
-			cerr << "Could not open output file \"" << outtrace << "\". Aborting" << endl;
+			cerr << "Could not open outputfile \"" << outtrace << "\". Aborting" << endl;
 
 		}
 
@@ -622,12 +625,12 @@ int main( int argc, char** argv ) {
 		assert(fha.mc);
 		if( 1 == OTF_MasterControl_read( fha.mc, intrace.c_str() ) ) {
 			if( 0 == OTF_MasterControl_write( fha.mc, outtrace.c_str() ) ) {
-				cerr << "Could not write OTF Master Control File \"" << outtrace << ".otf\". Aborting" << endl;
-				abort();
+				cerr << "Could not write Master Control File. Aborting" << endl;
+				exit(1);
 			}
 		} else {
-			cerr << "Could not read OTF Master Control File \"" << intrace << ".otf\". Aborting" << endl;
-			abort();
+			cerr << "Could not read Master Control File. Aborting" << endl;
+			exit(1);
 		}
 		
 		/* *** Read the Definitions:
@@ -718,7 +721,7 @@ int main( int argc, char** argv ) {
 			cerr << "Error while reading definitions. aborting" << endl;
 			OTF_WStream_close( fha.wstream );
 			OTF_RStream_close( defrstream );
-			abort();
+			exit(1);
 		}
 
 		OTF_WStream_close( fha.wstream );
@@ -743,7 +746,7 @@ int main( int argc, char** argv ) {
 				cerr << "Error while reading definitions. aborting" << endl;
 				OTF_WStream_close( fha.wstream );
 				OTF_RStream_close( defrstream );
-				abort();
+				exit(1);
 			}
 			
 			OTF_WStream_close( fha.wstream );
@@ -815,12 +818,11 @@ int main( int argc, char** argv ) {
 
 		//cerr << "parallel part " << endl;
 
-		
-		#ifdef VT_OMP
-		#pragma omp parallel for firstprivate(fha,nfiles,compression,minbytes,maxbytes) \
+#		if defined(HAVE_OMP) && HAVE_OMP
+#		pragma omp parallel for firstprivate(fha,nfiles,compression,minbytes,maxbytes) \
 			shared(curbytes,readrecords,lastprogressupdate,progress_counter) \
 			private(retev,retst,retsn)
-		#endif
+#		endif // HAVE_OMP
 		for( int streamindex = 0; streamindex < maxstreams; ++streamindex )
 		{
 
@@ -949,9 +951,9 @@ int main( int argc, char** argv ) {
 					OTF_RStream_eventBytesProgress( rstream,
 						&minbytestmp, &curbytestmp, &maxbytestmp );
 
-					#ifdef VT_OMP
-					#pragma omp critical (progressupdate)
-					#endif
+#					if defined(HAVE_OMP) && HAVE_OMP
+#					pragma omp critical (progressupdate)
+#					endif // HAVE_OMP
 					{
 						curbytes += curbytestmp - lastbytestmp;
 						lastbytestmp = curbytestmp;
@@ -979,9 +981,9 @@ int main( int argc, char** argv ) {
 					OTF_RStream_statisticBytesProgress( rstream,
 						&minbytestmp, &curbytestmp, &maxbytestmp );
 
-					#ifdef VT_OMP
-					#pragma omp critical (progressupdate)
-					#endif
+#					if defined(HAVE_OMP) && HAVE_OMP
+#					pragma omp critical (progressupdate)
+#					endif // HAVE_OMP
 					{
 						curbytes += curbytestmp - lastbytestmp;
 						lastbytestmp = curbytestmp;
@@ -1007,9 +1009,9 @@ int main( int argc, char** argv ) {
 					OTF_RStream_snapshotBytesProgress( rstream,
 							&minbytestmp, &curbytestmp, &maxbytestmp );
 
-					#ifdef VT_OMP
-					#pragma omp critical (progressupdate)
-					#endif
+#					if defined(HAVE_OMP) && HAVE_OMP
+#					pragma omp critical (progressupdate)
+#					endif // HAVE_OMP
 					{
 						curbytes += curbytestmp - lastbytestmp;
 						lastbytestmp = curbytestmp;
@@ -1195,6 +1197,7 @@ static map<uint32_t,uint64_t> readFilterFile( const string& filename, const map<
             line= line.substr(0, a);
             sline= new char[line.length()+1];
             strncpy( sline, line.c_str(), line.length() );
+            sline[line.length()] = '\0';
 
             char* token = strtok(sline, ";");
 			while( token ) {
