@@ -26,7 +26,8 @@ typedef enum {
   VT_INSTALLDIR_INCLUDEDIR,
   VT_INSTALLDIR_LIBDIR,
   VT_INSTALLDIR_DATADIR,
-  VT_INSTALLDIR_DATAROOTDIR
+  VT_INSTALLDIR_DATAROOTDIR,
+  VT_INSTALLDIR_SYSCONFDIR
 } VTInstallDirT;
 
 EXTERN char* vt_installdirs_get( VTInstallDirT type );
