@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2009 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -27,18 +27,24 @@
 #define OMPI_DEBUGGERS_H
 
 #include "ompi_config.h"
+#include "orte/mca/debugger/base/base.h"
 
 BEGIN_C_DECLS
 
     /**
      * Wait for a debugger if asked.
      */
-    OMPI_DECLSPEC void ompi_wait_for_debugger(void);
+    extern void ompi_wait_for_debugger(void);
 
     /**
      * Notify a debugger that we're about to abort
      */
-    OMPI_DECLSPEC void ompi_debugger_notify_abort(char *string);
+    extern void ompi_debugger_notify_abort(char *string);
+
+    /**
+     * Breakpoint function for parallel debuggers.
+     */
+    ORTE_DECLSPEC extern void *MPIR_Breakpoint(void);
 
 END_C_DECLS
 
