@@ -22,12 +22,9 @@
 #include "ompi_config.h"
 #include "opal/prefetch.h"
 #include "opal/util/opal_environ.h"
-#include "opal/util/if.h"
-#include "opal/util/argv.h"
 #include "ompi/constants.h"
 
 #include "opal/mca/base/mca_base_param.h"
-#include "orte/mca/errmgr/errmgr.h"
 #include "ompi/runtime/ompi_module_exchange.h"
 #include "ompi/mca/btl/base/btl_base_error.h"
 #include "ompi/mca/common/mx/common_mx.h"
@@ -347,7 +344,7 @@ static mca_btl_mx_module_t* mca_btl_mx_create(uint32_t board_num)
 
     mx_btl = malloc(sizeof(mca_btl_mx_module_t));
     if( NULL == mx_btl ) {
-        opal_output( 0, "mca_btl_mx_init: unable to allocate %d bytes of memory\n",
+        opal_output( 0, "mca_btl_mx_init: unable to allocate %lu bytes of memory\n",
                      sizeof(mca_btl_mx_module_t) );
         mx_close_endpoint(mx_endpoint);
         return NULL;
@@ -470,9 +467,9 @@ mca_btl_base_module_t** mca_btl_mx_component_init(int *num_btl_modules,
 
     ompi_free_list_init_new( &mca_btl_mx_component.mx_send_eager_frags,
                          sizeof(mca_btl_mx_frag_t) + mca_btl_mx_module.super.btl_eager_limit,
-                         CACHE_LINE_SIZE,
+                         opal_cache_line_size,
                          OBJ_CLASS(mca_btl_mx_frag_t),
-                         0,CACHE_LINE_SIZE,
+                         0,opal_cache_line_size,
                          mca_btl_mx_component.mx_free_list_num,
                          mca_btl_mx_component.mx_free_list_max,
                          mca_btl_mx_component.mx_free_list_inc,
@@ -480,9 +477,9 @@ mca_btl_base_module_t** mca_btl_mx_component_init(int *num_btl_modules,
 
     ompi_free_list_init_new( &mca_btl_mx_component.mx_send_user_frags,
                          sizeof(mca_btl_mx_frag_t),
-                         CACHE_LINE_SIZE,
+                         opal_cache_line_size,
                          OBJ_CLASS(mca_btl_mx_frag_t),
-                         0,CACHE_LINE_SIZE,
+                         0,opal_cache_line_size,
                          mca_btl_mx_component.mx_free_list_num,
                          mca_btl_mx_component.mx_free_list_max,
                          mca_btl_mx_component.mx_free_list_inc,
@@ -525,7 +522,7 @@ mca_btl_base_module_t** mca_btl_mx_component_init(int *num_btl_modules,
         }
         mx_addrs[count].unique_network_id = mx_btl->mx_unique_network_id;
 
-#if OMPI_ENABLE_HETEROGENEOUS_SUPPORT
+#if OPAL_ENABLE_HETEROGENEOUS_SUPPORT
         BTL_MX_ADDR_HTON(mx_addrs[count]);
 #endif
         mca_btl_mx_component.mx_btls[count] = mx_btl;
