@@ -13,6 +13,7 @@
  * Copyright (c) 2006-2009 Los Alamos National Security, LLC.  All rights
  *                         reserved.
  * Copyright (c) 2006-2007 Voltaire All rights reserved.
+ * Copyright (c) 2010      Oracle and/or its affiliates.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -28,9 +29,7 @@
 #include <infiniband/verbs.h>
 #include "ompi/mca/btl/btl.h"
 
-#if defined(c_plusplus) || defined(__cplusplus)
-extern "C" {
-#endif
+BEGIN_C_DECLS
 
 struct mca_btl_openib_reg_t;
 
@@ -73,7 +72,7 @@ typedef struct mca_btl_openib_header_coalesced_t {
      } while(0)
 
 struct mca_btl_openib_footer_t {
-#if OMPI_ENABLE_DEBUG
+#if OPAL_ENABLE_DEBUG
     uint32_t seq;
 #endif
     union {
@@ -94,7 +93,7 @@ typedef struct mca_btl_openib_footer_t mca_btl_openib_footer_t;
     } while (0)
 #endif
 
-#if OMPI_ENABLE_DEBUG
+#if OPAL_ENABLE_DEBUG
 #define BTL_OPENIB_FOOTER_SEQ_HTON(h)  ((h).seq = htonl((h).seq))
 #define BTL_OPENIB_FOOTER_SEQ_NTOH(h)  ((h).seq = ntohl((h).seq))
 #else
@@ -118,9 +117,13 @@ typedef struct mca_btl_openib_footer_t mca_btl_openib_footer_t;
 #define MCA_BTL_OPENIB_CONTROL_RDMA         1
 #define MCA_BTL_OPENIB_CONTROL_COALESCED    2
 #define MCA_BTL_OPENIB_CONTROL_CTS          3
+#if BTL_OPENIB_FAILOVER_ENABLED
+#define MCA_BTL_OPENIB_CONTROL_EP_BROKEN    4
+#define MCA_BTL_OPENIB_CONTROL_EP_EAGER_RDMA_ERROR 5
+#endif
 
 struct mca_btl_openib_control_header_t {
-    uint8_t type;
+    uint8_t  type;
 #if OMPI_OPENIB_PAD_HDR
     uint8_t  padding[15];
 #endif
@@ -165,6 +168,33 @@ do {                                               \
     (h).rdma_credits = ntohs((h).rdma_credits);    \
 } while (0)
 
+#if BTL_OPENIB_FAILOVER_ENABLED
+struct mca_btl_openib_broken_connection_header_t {
+    mca_btl_openib_control_header_t control;
+    uint8_t padding[3];
+    uint32_t lid;
+    uint64_t subnet_id;
+    uint32_t vpid;
+    uint32_t index; /* for eager RDMA only */
+};
+typedef struct mca_btl_openib_broken_connection_header_t mca_btl_openib_broken_connection_header_t;
+
+#define BTL_OPENIB_BROKEN_CONNECTION_HEADER_HTON(h)    \
+    do {                                               \
+        (h).lid = htonl((h).lid);                      \
+        (h).subnet_id = hton64((h).subnet_id);         \
+        (h).vpid = htonl((h).vpid);                    \
+        (h).index = htonl((h).index);                  \
+    } while (0)
+
+#define BTL_OPENIB_BROKEN_CONNECTION_HEADER_NTOH(h)    \
+    do {                                               \
+        (h).lid = ntohl((h).lid);                      \
+        (h).subnet_id = ntoh64((h).subnet_id);         \
+        (h).vpid = ntohl((h).vpid);                    \
+        (h).index = ntohl((h).index);                  \
+    } while (0)
+#endif
 enum mca_btl_openib_frag_type_t {
     MCA_BTL_OPENIB_FRAG_RECV,
     MCA_BTL_OPENIB_FRAG_RECV_USER,
@@ -347,7 +377,5 @@ typedef struct mca_btl_openib_frag_init_data_t mca_btl_openib_frag_init_data_t;
 void mca_btl_openib_frag_init(ompi_free_list_item_t* item, void* ctx);
 
 
-#if defined(c_plusplus) || defined(__cplusplus)
-}
-#endif
+END_C_DECLS
 #endif
