@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2007-2009 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2008      Mellanox Technologies. All rights reserved.
+ * Copyright (c) 2008-2009 Mellanox Technologies. All rights reserved.
  * Copyright (c) 2009      IBM Corporation.  All rights reserved.
  *
  * $COPYRIGHT$
@@ -259,10 +259,9 @@
 #include <fcntl.h>
 #include <infiniband/cm.h>
 
-#include "opal/util/if.h"
+#include "opal/util/output.h"
 #include "opal/util/error.h"
 #include "opal/event/event.h"
-#include "opal/class/opal_pointer_array.h"
 #include "opal_stdint.h"
 
 #include "orte/util/show_help.h"
@@ -543,7 +542,7 @@ ompi_btl_openib_connect_base_component_t ompi_btl_openib_connect_ibcm = {
 
 /*--------------------------------------------------------------------*/
 
-#define ENABLE_TIMERS (OMPI_ENABLE_DEBUG && 0)
+#define ENABLE_TIMERS (OPAL_ENABLE_DEBUG && 0)
 
 #if ENABLE_TIMERS 
 #include MCA_timer_IMPLEMENTATION_HEADER
@@ -655,7 +654,7 @@ static int ibcm_component_query(mca_btl_openib_module_t *btl,
        we're in an old version of OFED that is IB only (i.e., no
        iWarp), so we can safely assume that we can use this CPC. */
 #if defined(HAVE_STRUCT_IBV_DEVICE_TRANSPORT_TYPE)
-    if (IBV_TRANSPORT_IB != btl->device->ib_dev->transport_type) {
+    if (BTL_OPENIB_CONNECT_BASE_CHECK_IF_NOT_IB(btl)) {
         BTL_VERBOSE(("ibcm CPC only supported on InfiniBand; skipped on %s:%d",
                      ibv_get_device_name(btl->device->ib_dev),
                      openib_btl->port_num));
@@ -1012,8 +1011,9 @@ static int qp_create_all(mca_btl_base_endpoint_t* endpoint,
 
 
 /*
- * Fill in a path record for a peer.  For the moment, use the RDMA CM,
- * but someday we might just fill in the values ourselves.
+ * Fill in a path record for a peer.  This could be replaced with
+ * calls to rdma_resolve_addr() / rdma_resolve_route(), but for now,
+ * we fill it in with data we got from the modex, etc.
  */
 static int fill_path_record(ibcm_module_t *m,
                             mca_btl_base_endpoint_t *endpoint,
