@@ -9,11 +9,12 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2006-2008 Cisco, Inc.  All rights reserved.
+ * Copyright (c) 2006-2009 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2006      Los Alamos National Security, LLC.  All rights
  *                         reserved. 
- * Copyright (c) 2008      Mellanox Technologies.  All rights reserved.
- * Copyright (c) 2009-2011 IBM Corporation.  All rights reserved.
+ * Copyright (c) 2008-2011 Mellanox Technologies.  All rights reserved.
+ * Copyright (c) 2009      IBM Corporation.  All rights reserved.
+ * Copyright (c) 2010      Oracle and/or its affiliates.  All rights reserved
  *
  * $COPYRIGHT$
  * 
@@ -25,10 +26,12 @@
 #include "ompi_config.h"
 
 #include "opal/dss/dss.h"
+#include "opal_stdint.h"
 #include "orte/util/show_help.h"
 #include "opal/util/error.h"
-#include "orte/mca/oob/base/base.h"
+#include "opal/util/output.h"
 #include "orte/mca/rml/rml.h"
+#include "orte/mca/rml/rml_types.h"
 #include "orte/mca/errmgr/errmgr.h"
 #include "orte/util/name_fns.h"
 #include "orte/runtime/orte_globals.h"
@@ -40,6 +43,12 @@
 #include "connect/connect.h"
 #include "orte/util/show_help.h"
 
+#if (ENABLE_DYNAMIC_SL)
+#include "connect/btl_openib_connect_sl.h"
+#endif
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 typedef enum {
     ENDPOINT_CONNECT_REQUEST,
     ENDPOINT_CONNECT_RESPONSE,
@@ -119,7 +128,7 @@ static int oob_component_query(mca_btl_openib_module_t *btl,
        transport_type member, then we must be < OFED v1.2, and
        therefore we must be IB. */   
 #if defined(HAVE_STRUCT_IBV_DEVICE_TRANSPORT_TYPE)
-    if (IBV_TRANSPORT_IB != btl->device->ib_dev->transport_type) {
+    if (BTL_OPENIB_CONNECT_BASE_CHECK_IF_NOT_IB(btl)) {
         opal_output_verbose(5, mca_btl_base_output,
                             "openib BTL: oob CPC only supported on InfiniBand; skipped on  %s:%d",
                             ibv_get_device_name(btl->device->ib_dev),
@@ -214,7 +223,9 @@ static int oob_component_finalize(void)
         orte_rml.recv_cancel(ORTE_NAME_WILDCARD, OMPI_RML_TAG_OPENIB);
         rml_recv_posted = false;
     }
-
+#if (ENABLE_DYNAMIC_SL)
+    btl_openib_connect_sl_finalize();
+#endif
     return OMPI_SUCCESS;
 }
 
@@ -259,14 +270,15 @@ static int reply_start_connect(mca_btl_openib_endpoint_t *endpoint,
 static int set_remote_info(mca_btl_base_endpoint_t* endpoint,
                            mca_btl_openib_rem_info_t* rem_info)
 {
-    /* Free up the memory pointed to by rem_qps before overwriting the pointer
-       in the following memcpy */
-    free(endpoint->rem_info.rem_qps);
-
     /* copy the rem_info stuff */
-    memcpy(&((mca_btl_openib_endpoint_t*) endpoint)->rem_info,
-           rem_info, sizeof(mca_btl_openib_rem_info_t));
-
+    memcpy(&((mca_btl_openib_endpoint_t*) endpoint)->rem_info, 
+           rem_info, sizeof(mca_btl_openib_rem_info_t)); 
+    
+    /* copy over the rem qp info */
+    memcpy(endpoint->rem_info.rem_qps,
+           rem_info->rem_qps, sizeof(mca_btl_openib_rem_qp_info_t) * 
+           mca_btl_openib_component.num_qps);
+    
     BTL_VERBOSE(("Setting QP info,  LID = %d", endpoint->rem_info.rem_lid));
     return ORTE_SUCCESS;
 
@@ -297,9 +309,24 @@ static int qp_connect_all(mca_btl_openib_endpoint_t *endpoint)
         attr.min_rnr_timer  = mca_btl_openib_component.ib_min_rnr_timer;
         attr.ah_attr.is_global     = 0;
         attr.ah_attr.dlid          = endpoint->rem_info.rem_lid;
-        attr.ah_attr.sl            = mca_btl_openib_component.ib_service_level;
         attr.ah_attr.src_path_bits = openib_btl->src_path_bits;
         attr.ah_attr.port_num      = openib_btl->port_num;
+        attr.ah_attr.sl            = mca_btl_openib_component.ib_service_level;
+
+#if (ENABLE_DYNAMIC_SL)
+        /* if user enabled dynamic SL, get it from PathRecord */
+        if (0 != mca_btl_openib_component.ib_path_record_service_level) {
+            int rc = btl_openib_connect_get_pathrecord_sl(qp->context,
+                                                          attr.ah_attr.port_num,
+                                                          openib_btl->lid,
+                                                          attr.ah_attr.dlid);
+            if (OMPI_ERROR == rc) {
+                return OMPI_ERROR;
+            }
+            attr.ah_attr.sl = rc;
+        }
+#endif
+
         /* JMS to be filled in later dynamically */
         attr.ah_attr.static_rate   = 0;
 
@@ -595,7 +622,7 @@ static int send_connect_data(mca_btl_base_endpoint_t* endpoint,
         ORTE_ERROR_LOG(rc);
         return rc;
     }
-    BTL_VERBOSE(("Sent QP Info, LID = %d, SUBNET = %016x\n",
+    BTL_VERBOSE(("Sent QP Info, LID = %d, SUBNET = %" PRIx64 "\n",
                  endpoint->endpoint_btl->lid, 
                  endpoint->subnet_id));
 
@@ -634,12 +661,7 @@ static void rml_recv_cb(int status, orte_process_name_t* process_name,
     mca_btl_openib_rem_info_t rem_info;
     uint8_t message_type;
     bool master;
-
-    /* We later memcpy this whole structure. Make sure
-       that all the parameters are initialized, especially
-       the pointers */
-    memset(&rem_info,0, sizeof(rem_info));
-
+    
     /* start by unpacking data first so we know who is knocking at 
        our door */ 
     BTL_VERBOSE(("unpacking %d of %d\n", cnt, OPAL_UINT8));
@@ -724,7 +746,7 @@ static void rml_recv_cb(int status, orte_process_name_t* process_name,
         }
     }
     
-    BTL_VERBOSE(("Received QP Info,  LID = %d, SUBNET = %016x\n",
+    BTL_VERBOSE(("Received QP Info,  LID = %d, SUBNET = %" PRIx64 "\n",
                  rem_info.rem_lid, 
                  rem_info.rem_subnet_id));
     
@@ -765,6 +787,8 @@ static void rml_recv_cb(int status, orte_process_name_t* process_name,
                slave try to find  endpoint in closed state and
                initiate connection back */
             mca_btl_openib_endpoint_t *ib_endpoint_found = NULL;
+            int master_first_closed = -1;
+
             for (i = 0; i < ib_proc->proc_endpoint_count; i++) { 
                 ib_endpoint = ib_proc->proc_endpoints[i];
                 if (ib_endpoint->subnet_id != rem_info.rem_subnet_id ||
@@ -773,6 +797,13 @@ static void rml_recv_cb(int status, orte_process_name_t* process_name,
                     continue;
                 found = true;
                 ib_endpoint_found = ib_endpoint;
+
+                if (master && -1 == master_first_closed &&
+                    MCA_BTL_IB_CLOSED == ib_endpoint->endpoint_state ) {
+                    /* capture in case no endpoint in connecting state */
+                    master_first_closed = i;
+                }
+
                 if ((master &&
                      MCA_BTL_IB_CONNECTING == ib_endpoint->endpoint_state) ||
                     (!master &&
@@ -781,12 +812,20 @@ static void rml_recv_cb(int status, orte_process_name_t* process_name,
             }
             ib_endpoint = ib_endpoint_found;
             
+            if (found && master &&
+                MCA_BTL_IB_CLOSED == ib_endpoint->endpoint_state ) {
+                /* since this is master and no endpoints found in
+                 * connecting state use the first endpoint found
+                 * in closed state */
+                ib_endpoint = ib_proc->proc_endpoints[master_first_closed];
+            }
+
             /* if this is slave and there is no endpoints in closed
                state then all connection are already in progress so
                just ignore this connection request */
             if (found && !master &&
                 MCA_BTL_IB_CLOSED != ib_endpoint->endpoint_state) {
-                OPAL_THREAD_UNLOCK(&mca_btl_openib_component.ib_lock);
+		OPAL_THREAD_UNLOCK(&mca_btl_openib_component.ib_lock);
                 return;
             }
         }
@@ -794,7 +833,7 @@ static void rml_recv_cb(int status, orte_process_name_t* process_name,
         if (!found) {
             BTL_ERROR(("can't find suitable endpoint for this peer\n")); 
             mca_btl_openib_endpoint_invoke_error(NULL);
-            OPAL_THREAD_UNLOCK(&mca_btl_openib_component.ib_lock);
+	    OPAL_THREAD_UNLOCK(&mca_btl_openib_component.ib_lock);
             return; 
         }
         
@@ -809,7 +848,6 @@ static void rml_recv_cb(int status, orte_process_name_t* process_name,
                to CONNECTING, and then reply with our QP
                information */
             if (master) {
-                assert(rem_info.rem_qps != NULL);
                 rc = reply_start_connect(ib_endpoint, &rem_info);
             } else {
                 rc = oob_module_start_connect(ib_endpoint->endpoint_local_cpc, 
@@ -832,7 +870,6 @@ static void rml_recv_cb(int status, orte_process_name_t* process_name,
             break;
              
         case MCA_BTL_IB_CONNECTING :
-            assert(rem_info.rem_qps != NULL);
             set_remote_info(ib_endpoint, &rem_info);
             if (OMPI_SUCCESS != (rc = qp_connect_all(ib_endpoint))) {
                 BTL_ERROR(("endpoint connect error: %d", rc)); 
