@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
+ * Copyright (c) 2004-2010 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2008 The University of Tennessee and The University
+ * Copyright (c) 2004-2009 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2007 High Performance Computing Center Stuttgart,
@@ -10,6 +10,7 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2006-2007 Voltaire. All rights reserved.
+ * Copyright (c) 2009      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -26,25 +27,27 @@
 #include <fcntl.h>
 #include <errno.h>
 
-#include "opal/threads/mutex.h"
-#include "ompi/datatype/convertor.h"
 #include "opal/sys/atomic.h"
-#include "orte/util/show_help.h"
-#include "opal/util/if.h"
+#include "opal/class/opal_bitmap.h"
+#include "opal/util/output.h"
+#include "opal/util/printf.h"
 #include "opal/mca/carto/carto.h"
 #include "opal/mca/carto/base/base.h"
 #include "opal/mca/paffinity/base/base.h"
 #include "opal/mca/maffinity/base/base.h"
 #include "orte/util/proc_info.h"
-#include "opal/util/printf.h"
+#include "opal/datatype/opal_convertor.h"
 #include "ompi/class/ompi_free_list.h"
-#include "ompi/mca/pml/pml.h"
 #include "ompi/mca/btl/btl.h"
 #include "ompi/mca/mpool/base/base.h"
 #include "ompi/mca/common/sm/common_sm_mmap.h"
 #include "ompi/mca/mpool/sm/mpool_sm.h"
 
-#if OPAL_ENABLE_FT    == 1
+#if OMPI_BTL_SM_HAVE_KNEM
+#include <knem_io.h>
+#endif
+
+#if OPAL_ENABLE_FT_CR    == 1
 #include "opal/mca/crs/base/base.h"
 #include "opal/util/basename.h"
 #include "ompi/runtime/ompi_cr.h"
@@ -76,11 +79,15 @@ mca_btl_sm_t mca_btl_sm = {
         mca_btl_sm_alloc,
         mca_btl_sm_free,
         mca_btl_sm_prepare_src,
+#if OMPI_BTL_SM_HAVE_KNEM
+        mca_btl_sm_prepare_dst,
+#else
         NULL,
+#endif  /* OMPI_BTL_SM_HAVE_KNEM */
         mca_btl_sm_send,
-        NULL /*mca_btl_sm_sendi*/,  /* send immediate */
+        mca_btl_sm_sendi,
         NULL,  /* put */
-        NULL,  /* get */
+        NULL,  /* get -- optionally filled during initialization */
         mca_btl_base_dump,
         NULL, /* mpool */
         mca_btl_sm_register_error_cb, /* register error */
@@ -106,7 +113,7 @@ static void *mpool_calloc(size_t nmemb, size_t size)
     size_t bsize = nmemb * size;
     mca_mpool_base_module_t *mpool = mca_btl_sm_component.sm_mpool;
 
-    buf = mpool->mpool_alloc(mpool, bsize, CACHE_LINE_SIZE, 0, NULL);
+    buf = mpool->mpool_alloc(mpool, bsize, opal_cache_line_size, 0, NULL);
 
     if (NULL == buf)
         return NULL;
@@ -117,7 +124,7 @@ static void *mpool_calloc(size_t nmemb, size_t size)
 
 static void init_maffinity(int *my_mem_node, int *max_mem_node)
 {
-    static opal_carto_graph_t *topo;
+    opal_carto_graph_t *topo;
     opal_value_array_t dists;
     int i, num_core, socket;
     opal_paffinity_base_cpu_set_t cpus;
@@ -181,7 +188,7 @@ static int sm_btl_first_time_init(mca_btl_sm_t *sm_btl, int n)
 {
     size_t size, length, length_payload;
     char *sm_ctl_file;
-    mca_btl_sm_fifo_t *my_fifos;
+    sm_fifo_t *my_fifos;
     int my_mem_node=-1, num_mem_nodes=-1, i;
     ompi_proc_t **procs;
     size_t num_procs;
@@ -206,22 +213,29 @@ static int sm_btl_first_time_init(mca_btl_sm_t *sm_btl, int n)
         /*
          * This heuristic formula mostly says that we request memory for:
          * - nfifos FIFOs, each comprising:
-         *   . an mca_btl_sm_fifo_t structure
+         *   . a sm_fifo_t structure
          *   . many pointers (fifo_size of them per FIFO)
          * - eager fragments (2*n of them, allocated in sm_free_list_inc chunks)
          * - max fragments (sm_free_list_num of them)
          *
-         * On top of all that, we sprinkle in some number of "CACHE_LINE_SIZE"
+         * On top of all that, we sprinkle in some number of "opal_cache_line_size"
          * additions to account for some padding and edge effects that may lie
          * in the allocator.
          */
-        res.size = m->nfifos * ( sizeof(mca_btl_sm_fifo_t) + sizeof(void *) * m->fifo_size + 4 * CACHE_LINE_SIZE )
-            + ( 2 * n + m->sm_free_list_inc ) * ( m->eager_limit   + 2 * CACHE_LINE_SIZE )
-            +           m->sm_free_list_num   * ( m->max_frag_size + 2 * CACHE_LINE_SIZE );
-        if ( ((double) res.size) * n > LONG_MAX )
-            res.size  = LONG_MAX;
-        else
-            res.size *= n;
+        res.size =
+            FIFO_MAP_NUM(n) * ( sizeof(sm_fifo_t) + sizeof(void *) * m->fifo_size + 4 * opal_cache_line_size )
+            + ( 2 * n + m->sm_free_list_inc ) * ( m->eager_limit   + 2 * opal_cache_line_size )
+            +           m->sm_free_list_num   * ( m->max_frag_size + 2 * opal_cache_line_size );
+
+        /* before we multiply by n, make sure the result won't overflow */
+        /* Stick that little pad in, particularly since we'll eventually
+         * need a little extra space.  E.g., in mca_mpool_sm_init() in
+         * mpool_sm_component.c when sizeof(mca_common_sm_mmap_t) is
+         * added.
+         */
+        if ( ((double) res.size) * n > LONG_MAX - 4096 )
+            return OMPI_ERR_OUT_OF_RESOURCE;
+        res.size *= n;
 
         /* now, create it */
         mca_btl_sm_component.sm_mpools[i] =
@@ -239,11 +253,6 @@ static int sm_btl_first_time_init(mca_btl_sm_t *sm_btl, int n)
     mca_btl_sm_component.sm_mpool_base =
         mca_btl_sm_component.sm_mpools[0]->mpool_base(mca_btl_sm_component.sm_mpools[0]);
 
-    /* set the shared memory offset */
-    mca_btl_sm_component.sm_offset = (ptrdiff_t*)calloc(n, sizeof(ptrdiff_t));
-    if(NULL == mca_btl_sm_component.sm_offset)
-        return OMPI_ERR_OUT_OF_RESOURCE;
-
     /* create a list of peers */
     mca_btl_sm_component.sm_peers = (struct mca_btl_base_endpoint_t**)
         calloc(n, sizeof(struct mca_btl_base_endpoint_t*));
@@ -262,12 +271,12 @@ static int sm_btl_first_time_init(mca_btl_sm_t *sm_btl, int n)
     /* Pass in a data segment alignment of 0 to get no data
        segment (only the shared control structure) */
     size = sizeof(mca_common_sm_file_header_t) +
-        n * (sizeof(mca_btl_sm_fifo_t*) + sizeof(char *) + sizeof(uint16_t)) + CACHE_LINE_SIZE;
+        n * (sizeof(sm_fifo_t*) + sizeof(char *) + sizeof(uint16_t)) + opal_cache_line_size;
     procs = ompi_proc_world(&num_procs);
     if (!(mca_btl_sm_component.mmap_file =
           mca_common_sm_mmap_init(procs, num_procs, size, sm_ctl_file,
                                   sizeof(mca_common_sm_file_header_t),
-                                  CACHE_LINE_SIZE))) {
+                                  opal_cache_line_size))) {
         opal_output(0, "mca_btl_sm_add_procs: unable to create shared memory "
                     "BTL coordinating strucure :: size %lu \n",
                     (unsigned long)size);
@@ -291,24 +300,10 @@ static int sm_btl_first_time_init(mca_btl_sm_t *sm_btl, int n)
         return OMPI_ERROR;
     }
 
-    mca_btl_sm_component.shm_fifo = (mca_btl_sm_fifo_t **)mca_btl_sm_component.mmap_file->data_addr;
+    mca_btl_sm_component.shm_fifo = (volatile sm_fifo_t **)mca_btl_sm_component.mmap_file->data_addr;
     mca_btl_sm_component.shm_bases = (char**)(mca_btl_sm_component.shm_fifo + n);
     mca_btl_sm_component.shm_mem_nodes = (uint16_t*)(mca_btl_sm_component.shm_bases + n);
 
-    /* Sync with other local procs. (Do we have to?) */
-    if(0 == mca_btl_sm_component.my_smp_rank) {
-        mca_btl_sm_component.mmap_file->map_seg->seg_inited = true;
-
-        /* memory barrier to ensure this flag is set before other
-         *  flags are set */
-        opal_atomic_wmb();
-    } else {
-        while(!mca_btl_sm_component.mmap_file->map_seg->seg_inited) {
-            opal_atomic_rmb();
-            opal_progress();
-        }
-    }
-
     /* set the base of the shared memory segment */
     mca_btl_sm_component.shm_bases[mca_btl_sm_component.my_smp_rank] =
         (char*)mca_btl_sm_component.sm_mpool_base;
@@ -316,16 +311,14 @@ static int sm_btl_first_time_init(mca_btl_sm_t *sm_btl, int n)
         (uint16_t)my_mem_node;
 
     /* initialize the array of fifo's "owned" by this process */
-    if(NULL == (my_fifos = (mca_btl_sm_fifo_t*)mpool_calloc(FIFO_MAP_NUM(n), sizeof(mca_btl_sm_fifo_t))))
+    if(NULL == (my_fifos = (sm_fifo_t*)mpool_calloc(FIFO_MAP_NUM(n), sizeof(sm_fifo_t))))
         return OMPI_ERR_OUT_OF_RESOURCE;
 
     mca_btl_sm_component.shm_fifo[mca_btl_sm_component.my_smp_rank] = my_fifos;
 
-    opal_atomic_wmb();
-
     /* cache the pointer to the 2d fifo array.  These addresses
      * are valid in the current process space */
-    mca_btl_sm_component.fifo = (mca_btl_sm_fifo_t**)malloc(sizeof(mca_btl_sm_fifo_t*) * n);
+    mca_btl_sm_component.fifo = (sm_fifo_t**)malloc(sizeof(sm_fifo_t*) * n);
 
     if(NULL == mca_btl_sm_component.fifo)
         return OMPI_ERR_OUT_OF_RESOURCE;
@@ -343,8 +336,8 @@ static int sm_btl_first_time_init(mca_btl_sm_t *sm_btl, int n)
     length_payload =
         sizeof(mca_btl_sm_hdr_t) + mca_btl_sm_component.eager_limit;
     i = ompi_free_list_init_new(&mca_btl_sm_component.sm_frags_eager, length,
-                                CACHE_LINE_SIZE, OBJ_CLASS(mca_btl_sm_frag1_t),
-                                length_payload, CACHE_LINE_SIZE,
+                                opal_cache_line_size, OBJ_CLASS(mca_btl_sm_frag1_t),
+                                length_payload, opal_cache_line_size,
                                 mca_btl_sm_component.sm_free_list_num,
                                 mca_btl_sm_component.sm_free_list_max,
                                 mca_btl_sm_component.sm_free_list_inc,
@@ -356,8 +349,8 @@ static int sm_btl_first_time_init(mca_btl_sm_t *sm_btl, int n)
     length_payload =
         sizeof(mca_btl_sm_hdr_t) + mca_btl_sm_component.max_frag_size;
     i = ompi_free_list_init_new(&mca_btl_sm_component.sm_frags_max, length,
-                                CACHE_LINE_SIZE, OBJ_CLASS(mca_btl_sm_frag2_t),
-                                length_payload, CACHE_LINE_SIZE,
+                                opal_cache_line_size, OBJ_CLASS(mca_btl_sm_frag2_t),
+                                length_payload, opal_cache_line_size,
                                 mca_btl_sm_component.sm_free_list_num,
                                 mca_btl_sm_component.sm_free_list_max,
                                 mca_btl_sm_component.sm_free_list_inc,
@@ -365,11 +358,22 @@ static int sm_btl_first_time_init(mca_btl_sm_t *sm_btl, int n)
     if ( OMPI_SUCCESS != i )
         return i;
 
+    i = ompi_free_list_init_new(&mca_btl_sm_component.sm_frags_user, 
+		    sizeof(mca_btl_sm_user_t),
+		    opal_cache_line_size, OBJ_CLASS(mca_btl_sm_user_t),
+		    sizeof(mca_btl_sm_hdr_t), opal_cache_line_size,
+		    mca_btl_sm_component.sm_free_list_num,
+		    mca_btl_sm_component.sm_free_list_max,
+		    mca_btl_sm_component.sm_free_list_inc,
+		    mca_btl_sm_component.sm_mpool);
+    if ( OMPI_SUCCESS != i )
+	    return i;   
+
     mca_btl_sm_component.num_outstanding_frags = 0;
 
     mca_btl_sm_component.num_pending_sends = 0;
     i = opal_free_list_init(&mca_btl_sm_component.pending_send_fl,
-                            sizeof(mca_btl_sm_pending_send_item_t),
+                            sizeof(btl_sm_pending_send_item_t),
                             OBJ_CLASS(opal_free_list_item_t),
                             16, -1, 32);
     if ( OMPI_SUCCESS != i )
@@ -385,7 +389,7 @@ static struct mca_btl_base_endpoint_t *
 create_sm_endpoint(int local_proc, struct ompi_proc_t *proc)
 {
     struct mca_btl_base_endpoint_t *ep;
-#if OMPI_ENABLE_PROGRESS_THREADS == 1
+#if OPAL_ENABLE_PROGRESS_THREADS == 1
     char path[PATH_MAX];
 #endif
 
@@ -396,7 +400,7 @@ create_sm_endpoint(int local_proc, struct ompi_proc_t *proc)
     ep->peer_smp_rank = local_proc + mca_btl_sm_component.num_smp_procs;
 
     OBJ_CONSTRUCT(&ep->pending_sends, opal_list_t);
-#if OMPI_ENABLE_PROGRESS_THREADS == 1
+#if OPAL_ENABLE_PROGRESS_THREADS == 1
     sprintf(path, "%s"OPAL_PATH_SEP"sm_fifo.%lu",
             orte_process_info.job_session_dir,
             (unsigned long)proc->proc_name.vpid);
@@ -432,11 +436,10 @@ int mca_btl_sm_add_procs(
     size_t nprocs,
     struct ompi_proc_t **procs,
     struct mca_btl_base_endpoint_t **peers,
-    ompi_bitmap_t* reachability)
+    opal_bitmap_t* reachability)
 {
     int return_code = OMPI_SUCCESS;
-    int32_t n_local_procs = 0, proc, j,
-        my_smp_rank = mca_btl_sm_component.my_smp_rank;
+    int32_t n_local_procs = 0, proc, j, my_smp_rank = -1;
     ompi_proc_t* my_proc; /* pointer to caller's proc structure */
     mca_btl_sm_t *sm_btl;
     bool have_connected_peer = false;
@@ -457,7 +460,7 @@ int mca_btl_sm_add_procs(
         /* check to see if this proc can be reached via shmem (i.e.,
            if they're on my local host and in my job) */
         if (procs[proc]->proc_name.jobid != my_proc->proc_name.jobid ||
-                 0 == (procs[proc]->proc_flags & OMPI_PROC_FLAG_LOCAL)) {
+            !OPAL_PROC_ON_LOCAL_NODE(procs[proc]->proc_flags)) {
             peers[proc] = NULL;
             continue;
         }
@@ -478,7 +481,7 @@ int mca_btl_sm_add_procs(
         n_local_procs++;
 
         /* add this proc to shared memory accessibility list */
-        return_code = ompi_bitmap_set_bit(reachability, proc);
+        return_code = opal_bitmap_set_bit(reachability, proc);
         if(OMPI_SUCCESS != return_code)
             goto CLEANUP;
     }
@@ -503,7 +506,7 @@ int mca_btl_sm_add_procs(
     }
 
     /* set local proc's smp rank in the peers structure for
-     * rapid access and calulcate reachebility */
+     * rapid access and calculate reachability */
     for(proc = 0; proc < (int32_t)nprocs; proc++) {
         if(NULL == peers[proc])
             continue;
@@ -530,6 +533,18 @@ int mca_btl_sm_add_procs(
             goto CLEANUP;
     }
 
+    opal_atomic_wmb();
+
+    /* Sync with other local procs. Force the FIFO initialization to always
+     * happens before the readers access it.
+     */
+    opal_atomic_add_32( &mca_btl_sm_component.mmap_file->map_seg->seg_inited, 1);
+    while( n_local_procs >
+           mca_btl_sm_component.mmap_file->map_seg->seg_inited) {
+        opal_progress();
+        opal_atomic_rmb();
+    }
+
     /* coordinate with other processes */
     for(j = mca_btl_sm_component.num_smp_procs;
         j < mca_btl_sm_component.num_smp_procs + n_local_procs; j++) {
@@ -537,18 +552,18 @@ int mca_btl_sm_add_procs(
 
         /* spin until this element is allocated */
         /* doesn't really wait for that process... FIFO might be allocated, but not initialized */
+        opal_atomic_rmb();
         while(NULL == mca_btl_sm_component.shm_fifo[j]) {
-            opal_atomic_rmb();
             opal_progress();
+            opal_atomic_rmb();
         }
 
         /* Calculate the difference as (my_base - their_base) */
         diff = ADDR2OFFSET(bases[my_smp_rank], bases[j]);
-        mca_btl_sm_component.sm_offset[j] = diff;
 
         /* store local address of remote fifos */
         mca_btl_sm_component.fifo[j] =
-            (mca_btl_sm_fifo_t*)OFFSET2ADDR(diff, mca_btl_sm_component.shm_fifo[j]);
+            (sm_fifo_t*)OFFSET2ADDR(diff, mca_btl_sm_component.shm_fifo[j]);
 
         /* cache local copy of peer memory node number */
         mca_btl_sm_component.mem_nodes[j] = mca_btl_sm_component.shm_mem_nodes[j];
@@ -662,7 +677,7 @@ struct mca_btl_base_descriptor_t* mca_btl_sm_prepare_src(
     struct mca_btl_base_module_t* btl,
     struct mca_btl_base_endpoint_t* endpoint,
     mca_mpool_base_registration_t* registration,
-    struct ompi_convertor_t* convertor,
+    struct opal_convertor_t* convertor,
     uint8_t order,
     size_t reserve,
     size_t* size,
@@ -673,30 +688,69 @@ struct mca_btl_base_descriptor_t* mca_btl_sm_prepare_src(
     uint32_t iov_count = 1;
     size_t max_data = *size;
     int rc;
+#if OMPI_BTL_SM_HAVE_KNEM
+    mca_btl_sm_t* sm_btl = (mca_btl_sm_t*)btl;
+    struct knem_cmd_create_region knem_cr;
+    struct knem_cmd_param_iovec knem_iov;
 
-    if ( reserve + max_data <= mca_btl_sm_component.eager_limit ) {
-        MCA_BTL_SM_FRAG_ALLOC_EAGER(frag,rc);
+    if( (0 != reserve) || (OPAL_UNLIKELY(!mca_btl_sm_component.use_knem)) ) {
+#endif
+        if ( reserve + max_data <= mca_btl_sm_component.eager_limit ) {
+            MCA_BTL_SM_FRAG_ALLOC_EAGER(frag,rc);
+        } else {
+            MCA_BTL_SM_FRAG_ALLOC_MAX(frag, rc);
+        }
+        if( OPAL_UNLIKELY(NULL == frag) ) {
+            return NULL;
+        }
+
+        if( OPAL_UNLIKELY(reserve + max_data > frag->size) ) {
+            max_data = frag->size - reserve;
+        }
+        iov.iov_len = max_data;
+        iov.iov_base =
+            (IOVBASE_TYPE*)(((unsigned char*)(frag->segment.seg_addr.pval)) +
+                            reserve);
+
+        rc = opal_convertor_pack(convertor, &iov, &iov_count, &max_data );
+        if( OPAL_UNLIKELY(rc < 0) ) {
+            MCA_BTL_SM_FRAG_RETURN(frag);
+            return NULL;
+        }
+        frag->segment.seg_len = reserve + max_data;
+#if OMPI_BTL_SM_HAVE_KNEM
     } else {
-        MCA_BTL_SM_FRAG_ALLOC_MAX(frag, rc);
-    }
-    if(OPAL_UNLIKELY(NULL == frag)) {
-        return NULL;
-    }
+        MCA_BTL_SM_FRAG_ALLOC_USER(frag, rc);
+        if( OPAL_UNLIKELY(NULL == frag) ) {
+            return NULL;
+        }
+        iov.iov_len = max_data;
+        iov.iov_base = NULL;
+        rc = opal_convertor_pack(convertor, &iov, &iov_count, &max_data);
+        if( OPAL_UNLIKELY(rc < 0) ) {
+            MCA_BTL_SM_FRAG_RETURN(frag);
+            return NULL;
+        }
+        frag->segment.seg_addr.pval = iov.iov_base;
+        frag->segment.seg_len = max_data;
 
-    if(reserve + max_data > frag->size) {
-        max_data = frag->size - reserve;
+        knem_iov.base = (uintptr_t)iov.iov_base;
+        knem_iov.len = max_data;
+        knem_cr.iovec_array = (uintptr_t)&knem_iov;
+        knem_cr.iovec_nr = iov_count;
+        knem_cr.protection = PROT_READ;
+        knem_cr.flags = KNEM_FLAG_SINGLEUSE;
+        if (OPAL_UNLIKELY(ioctl(sm_btl->knem_fd, KNEM_CMD_CREATE_REGION, &knem_cr) < 0)) {
+            return NULL;
+        }
+        frag->segment.seg_key.key64 = knem_cr.cookie;
     }
-    iov.iov_len = max_data;
-    iov.iov_base =
-        (IOVBASE_TYPE*)(((unsigned char*)(frag->segment.seg_addr.pval)) +
-                reserve);
-
-    rc = ompi_convertor_pack(convertor, &iov, &iov_count, &max_data );
-    if(rc < 0) {
-        MCA_BTL_SM_FRAG_RETURN(frag);
-        return NULL;
-    }
-    frag->segment.seg_len = reserve + max_data;
+#endif
+    frag->base.des_src = &(frag->segment);
+    frag->base.des_src_cnt = 1;
+    frag->base.order = MCA_BTL_NO_ORDER;
+    frag->base.des_dst = NULL;
+    frag->base.des_dst_cnt = 0;
     frag->base.des_flags = flags;
     *size = max_data;
     return &frag->base;
@@ -742,7 +796,7 @@ struct mca_btl_base_descriptor_t* mca_btl_sm_prepare_src(
  */
 int mca_btl_sm_sendi( struct mca_btl_base_module_t* btl,
                       struct mca_btl_base_endpoint_t* endpoint,
-                      struct ompi_convertor_t* convertor,
+                      struct opal_convertor_t* convertor,
                       void* header,
                       size_t header_size,
                       size_t payload_size,
@@ -751,7 +805,7 @@ int mca_btl_sm_sendi( struct mca_btl_base_module_t* btl,
                       mca_btl_base_tag_t tag,
                       mca_btl_base_descriptor_t** descriptor )
 {
-    size_t max_data, length = (header_size + payload_size);
+    size_t length = (header_size + payload_size);
     mca_btl_sm_frag_t* frag;
     int rc;
 
@@ -761,20 +815,32 @@ int mca_btl_sm_sendi( struct mca_btl_base_module_t* btl,
 
     /* this check should be unnecessary... turn into an assertion? */
     if( length < mca_btl_sm_component.eager_limit ) {
+
+        /* allocate a fragment, giving up if we can't get one */
+        /* note that frag==NULL is equivalent to rc returning an error code */
         MCA_BTL_SM_FRAG_ALLOC_EAGER(frag, rc);
         if( OPAL_UNLIKELY(NULL == frag) ) {
             *descriptor = NULL;
             return rc;
         }
+
+        /* fill in fragment fields */
         frag->segment.seg_len = length;
-        frag->hdr->len = length;
+        frag->hdr->len        = length;
         assert( 0 == (flags & MCA_BTL_DES_SEND_ALWAYS_CALLBACK) );
-        frag->base.des_flags = flags | MCA_BTL_DES_FLAGS_BTL_OWNERSHIP;
+        frag->base.des_flags = flags | MCA_BTL_DES_FLAGS_BTL_OWNERSHIP;   /* why do any flags matter here other than OWNERSHIP? */
         frag->hdr->tag = tag;
         frag->endpoint = endpoint;
 
+        /* write the match header (with MPI comm/tag/etc. info) */
         memcpy( frag->segment.seg_addr.pval, header, header_size );
+
+        /* write the message data if there is any */
+        /*
+          We can add MEMCHECKER calls before and after the packing.
+        */
         if( payload_size ) {
+            size_t max_data;
             struct iovec iov;
             uint32_t iov_count;
             /* pack the data into the supplied buffer */
@@ -782,21 +848,26 @@ int mca_btl_sm_sendi( struct mca_btl_base_module_t* btl,
             iov.iov_len  = max_data = payload_size;
             iov_count    = 1;
 
-            (void)ompi_convertor_pack( convertor,
-                                       &iov, &iov_count, &max_data);
+            (void)opal_convertor_pack( convertor, &iov, &iov_count, &max_data);
 
             assert(max_data == payload_size);
         }
+
         MCA_BTL_SM_TOUCH_DATA_TILL_CACHELINE_BOUNDARY(frag);
+
+        /* write the fragment pointer to the FIFO */
         /*
-         * post the descriptor in the queue - post with the relative
-         * address
+         * Note that we don't care what the FIFO-write return code is.  Even if
+         * the return code indicates failure, the write has still "completed" from
+         * our point of view:  it has been posted to a "pending send" queue.
          */
         OPAL_THREAD_ADD32(&mca_btl_sm_component.num_outstanding_frags, +1);
         MCA_BTL_SM_FIFO_WRITE(endpoint, endpoint->my_smp_rank,
                               endpoint->peer_smp_rank, (void *) VIRTUAL2RELATIVE(frag->hdr), false, true, rc);
-        return rc;
+        return OMPI_SUCCESS;
     }
+
+    /* presumably, this code path will never get executed */
     *descriptor = mca_btl_sm_alloc( btl, endpoint, order,
                                     payload_size + header_size, flags);
     return OMPI_ERR_RESOURCE_BUSY;
@@ -846,7 +917,181 @@ int mca_btl_sm_send( struct mca_btl_base_module_t* btl,
     return 0;
 }
 
-#if OPAL_ENABLE_FT    == 0
+#if OMPI_BTL_SM_HAVE_KNEM
+struct mca_btl_base_descriptor_t* mca_btl_sm_prepare_dst( 
+		struct mca_btl_base_module_t* btl,
+		struct mca_btl_base_endpoint_t* endpoint,
+		struct mca_mpool_base_registration_t* registration,
+		struct opal_convertor_t* convertor,
+		uint8_t order,
+		size_t reserve,
+		size_t* size,
+		uint32_t flags)
+{
+    int rc;
+    mca_btl_sm_frag_t* frag;
+
+    MCA_BTL_SM_FRAG_ALLOC_USER(frag, rc);
+    if(OPAL_UNLIKELY(NULL == frag)) {
+        return NULL;
+    }
+    
+    frag->segment.seg_len = *size;
+    opal_convertor_get_current_pointer( convertor, (void**)&(frag->segment.seg_addr.pval) );
+    
+    frag->base.des_src = NULL;
+    frag->base.des_src_cnt = 0;
+    frag->base.des_dst = &frag->segment;
+    frag->base.des_dst_cnt = 1;
+    frag->base.des_flags = flags;
+    return &frag->base;
+}
+
+/**
+ * Initiate an synchronous get.
+ *
+ * @param btl (IN)         BTL module
+ * @param endpoint (IN)    BTL addressing information
+ * @param descriptor (IN)  Description of the data to be transferred
+ */
+int mca_btl_sm_get_sync(struct mca_btl_base_module_t* btl,
+                        struct mca_btl_base_endpoint_t* endpoint,
+                        struct mca_btl_base_descriptor_t* des)
+{
+    int btl_ownership;
+    mca_btl_sm_t* sm_btl = (mca_btl_sm_t*) btl;
+    mca_btl_sm_frag_t* frag = (mca_btl_sm_frag_t*)des;
+    mca_btl_base_segment_t *src = des->des_src;
+    mca_btl_base_segment_t *dst = des->des_dst;
+    struct knem_cmd_inline_copy icopy;
+    struct knem_cmd_param_iovec recv_iovec;
+    
+    /* Fill in the ioctl data fields.  There's no async completion, so
+       we don't need to worry about getting a slot, etc. */
+    recv_iovec.base = (uintptr_t) dst->seg_addr.pval;
+    recv_iovec.len =  dst->seg_len;
+    icopy.local_iovec_array = (uintptr_t)&recv_iovec;
+    icopy.local_iovec_nr = 1;
+    icopy.remote_cookie = src->seg_key.key64;
+    icopy.remote_offset = 0;
+    icopy.write = 0;
+
+    /* Use the DMA flag if knem supports it *and* the segment length
+       is greater than the cutoff.  Note that if the knem_dma_min
+       value is 0 (i.e., the MCA param was set to 0), the segment size
+       will never be larger than it, so DMA will never be used. */
+    icopy.flags = 0;
+    if (mca_btl_sm_component.knem_dma_min <= dst->seg_len) {
+        icopy.flags = mca_btl_sm_component.knem_dma_flag;
+    }
+    /* synchronous flags only, no need to specify icopy.async_status_index */
+
+    /* When the ioctl returns, the transfer is done and we can invoke
+       the btl callback and return the frag */
+    if (OPAL_UNLIKELY(0 != ioctl(sm_btl->knem_fd, 
+                                 KNEM_CMD_INLINE_COPY, &icopy))) {
+        return OMPI_ERROR;
+    }
+
+    /* FIXME: what if icopy.current_status == KNEM_STATUS_FAILED? */
+
+    btl_ownership = (frag->base.des_flags & MCA_BTL_DES_FLAGS_BTL_OWNERSHIP);
+    if (0 != (MCA_BTL_DES_SEND_ALWAYS_CALLBACK & frag->base.des_flags)) {
+        frag->base.des_cbfunc(&mca_btl_sm.super, 
+                              frag->endpoint, &frag->base, 
+                              OMPI_SUCCESS);
+    }
+    if (btl_ownership) {
+        MCA_BTL_SM_FRAG_RETURN(frag);
+    }
+
+    return OMPI_SUCCESS;
+}
+
+
+/**
+ * Initiate an asynchronous get.
+ *
+ * @param btl (IN)         BTL module
+ * @param endpoint (IN)    BTL addressing information
+ * @param descriptor (IN)  Description of the data to be transferred
+ */
+int mca_btl_sm_get_async(struct mca_btl_base_module_t* btl,
+                         struct mca_btl_base_endpoint_t* endpoint,
+                         struct mca_btl_base_descriptor_t* des)
+{
+    int btl_ownership;
+    mca_btl_sm_t* sm_btl = (mca_btl_sm_t*) btl;
+    mca_btl_sm_frag_t* frag = (mca_btl_sm_frag_t*)des;
+    mca_btl_base_segment_t *src = des->des_src;
+    mca_btl_base_segment_t *dst = des->des_dst;
+    struct knem_cmd_inline_copy icopy;
+    struct knem_cmd_param_iovec recv_iovec;
+    
+    /* If we have no knem slots available, return
+       TEMP_OUT_OF_RESOURCE */
+    if (sm_btl->knem_status_num_used >=
+        mca_btl_sm_component.knem_max_simultaneous) {
+        return OMPI_ERR_TEMP_OUT_OF_RESOURCE;
+    }
+
+    /* We have a slot, so fill in the data fields.  Bump the
+       first_avail and num_used counters. */
+    recv_iovec.base = (uintptr_t) dst->seg_addr.pval;
+    recv_iovec.len =  dst->seg_len;
+    icopy.local_iovec_array = (uintptr_t)&recv_iovec;
+    icopy.local_iovec_nr = 1;
+    icopy.write = 0;
+    icopy.async_status_index = sm_btl->knem_status_first_avail++;
+    if (sm_btl->knem_status_first_avail >= 
+        mca_btl_sm_component.knem_max_simultaneous) {
+        sm_btl->knem_status_first_avail = 0;
+    }
+    ++sm_btl->knem_status_num_used;
+    icopy.remote_cookie = src->seg_key.key64;
+    icopy.remote_offset = 0;
+
+    /* Use the DMA flag if knem supports it *and* the segment length
+       is greater than the cutoff */
+    icopy.flags = KNEM_FLAG_ASYNCDMACOMPLETE;
+    if (mca_btl_sm_component.knem_dma_min <= dst->seg_len) {
+        icopy.flags = mca_btl_sm_component.knem_dma_flag;
+    }
+
+    sm_btl->knem_frag_array[icopy.async_status_index] = frag;
+    if (OPAL_LIKELY(0 == ioctl(sm_btl->knem_fd, 
+                               KNEM_CMD_INLINE_COPY, &icopy))) {
+        if (icopy.current_status != KNEM_STATUS_PENDING) {
+            /* request completed synchronously */
+
+            /* FIXME: what if icopy.current_status == KNEM_STATUS_FAILED? */
+
+            btl_ownership = (frag->base.des_flags & MCA_BTL_DES_FLAGS_BTL_OWNERSHIP);
+            if (0 != (MCA_BTL_DES_SEND_ALWAYS_CALLBACK & frag->base.des_flags)) {
+                frag->base.des_cbfunc(&mca_btl_sm.super, 
+                                      frag->endpoint, &frag->base, 
+                                      OMPI_SUCCESS);
+            }
+            if (btl_ownership) {
+                MCA_BTL_SM_FRAG_RETURN(frag);
+            }
+
+            --sm_btl->knem_status_num_used;
+            ++sm_btl->knem_status_first_used;
+            if (sm_btl->knem_status_first_used >=
+                mca_btl_sm_component.knem_max_simultaneous) {
+                sm_btl->knem_status_first_used = 0;
+            }
+        }
+        return OMPI_SUCCESS;
+    } else {
+        return OMPI_ERROR;
+    }
+}
+#endif
+
+
+#if OPAL_ENABLE_FT_CR    == 0
 int mca_btl_sm_ft_event(int state) {
     return OMPI_SUCCESS;
 }
@@ -913,4 +1158,4 @@ int mca_btl_sm_ft_event(int state) {
 
     return OMPI_SUCCESS;
 }
-#endif /* OPAL_ENABLE_FT */
+#endif /* OPAL_ENABLE_FT_CR */
