@@ -10,8 +10,6 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2006      Voltaire. All rights reserved.
- * Copyright (c) 2011      Los Alamos National Security, LLC. All rights
- *                         reserved.
  *
  * $COPYRIGHT$
  *
@@ -25,14 +23,13 @@
 #ifndef MCA_MPOOL_OPENIB_H
 #define MCA_MPOOL_OPENIB_H
 
+#include "ompi_config.h"
 #include "opal/class/opal_list.h"
 #include "ompi/class/ompi_free_list.h"
 #include "opal/event/event.h"
 #include "ompi/mca/mpool/mpool.h"
 
-#if defined(c_plusplus) || defined(__cplusplus)
-extern "C" {
-#endif
+BEGIN_C_DECLS
 
 struct mca_mpool_rdma_component_t {
     mca_mpool_base_component_t super;
@@ -58,7 +55,7 @@ struct mca_mpool_rdma_module_t {
     mca_mpool_base_module_t super;
     struct mca_mpool_base_resources_t resources;
     ompi_free_list_t reg_list;
-    opal_list_t lru_list;
+    opal_list_t mru_list;
     opal_list_t gc_list;
     uint32_t stat_cache_hit;
     uint32_t stat_cache_miss;
@@ -132,7 +129,5 @@ void mca_mpool_rdma_finalize(struct mca_mpool_base_module_t *mpool);
  */
 int mca_mpool_rdma_ft_event(int state);
 
-#if defined(c_plusplus) || defined(__cplusplus)
-}
-#endif
+END_C_DECLS
 #endif
