@@ -23,8 +23,7 @@
 #ifndef MCA_MPOOL_SM_H
 #define MCA_MPOOL_SM_H
 
-#include "opal/class/opal_list.h"
-#include "ompi/class/ompi_free_list.h"
+#include "ompi_config.h"
 
 #include "opal/event/event.h"
 
@@ -32,9 +31,7 @@
 #include "ompi/mca/mpool/mpool.h"
 #include "ompi/mca/allocator/allocator.h"
 
-#if defined(c_plusplus) || defined(__cplusplus)
-extern "C" {
-#endif
+BEGIN_C_DECLS
 
 struct mca_mpool_sm_component_t {
   mca_mpool_base_component_t super;
@@ -106,8 +103,6 @@ void mca_mpool_sm_free(
  */
 int mca_mpool_sm_ft_event(int state);
 
-#if defined(c_plusplus) || defined(__cplusplus)
-}
-#endif
+END_C_DECLS
 
 #endif
