@@ -19,10 +19,9 @@
 #include "ompi_config.h"
 
 #include "opal/mca/mca.h"
+#include "opal/util/output.h"
 #include "opal/mca/base/base.h"
-#include "orte/util/show_help.h"
 
-#include "orte/util/show_help.h"
 
 #include "ompi/constants.h"
 #include "ompi/mca/mtl/mtl.h"
