@@ -7,6 +7,7 @@
  *                         reserved.
  * Copyright (c) 2004-2006 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2010      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -19,7 +20,6 @@
 #include "pml_cm.h"
 #include "opal/event/event.h"
 #include "opal/mca/base/mca_base_param.h"
-#include "ompi/datatype/convertor.h"
 #include "ompi/mca/mtl/mtl.h"
 #include "ompi/mca/mtl/base/base.h"
 #include "ompi/mca/pml/base/pml_base_bsend.h"
@@ -28,6 +28,7 @@
 #include "pml_cm_recvreq.h"
 #include "pml_cm_component.h"
 
+static int mca_pml_cm_component_register(void);
 static int mca_pml_cm_component_open(void);
 static int mca_pml_cm_component_close(void);
 static mca_pml_base_module_t* mca_pml_cm_component_init( int* priority,
@@ -47,7 +48,9 @@ mca_pml_base_component_2_0_0_t mca_pml_cm_component = {
          OMPI_MINOR_VERSION,  /* MCA component minor version */
          OMPI_RELEASE_VERSION,  /* MCA component release version */
          mca_pml_cm_component_open,  /* component open */
-         mca_pml_cm_component_close  /* component close */
+         mca_pml_cm_component_close,  /* component close */
+         NULL,
+         mca_pml_cm_component_register,
      },
      {
          /* This component is not checkpoint ready */
@@ -71,12 +74,8 @@ void (*send_completion_callbacks[MCA_PML_BASE_SEND_SIZE])
     mca_pml_cm_send_request_completion } ;
 
 static int
-mca_pml_cm_component_open(void)
+mca_pml_cm_component_register(void)
 {
-    int ret;
-
-    ret = ompi_mtl_base_open();
-    if (OMPI_SUCCESS != ret) return ret;
 
     mca_base_param_reg_int(&mca_pml_cm_component.pmlm_version,
                            "free_list_num",
@@ -107,10 +106,26 @@ mca_pml_cm_component_open(void)
                            "CM PML selection priority",
                            false,
                            false,
-                           30,
+                           10,
                            &ompi_pml_cm.default_priority);
 
-    return OMPI_SUCCESS;
+    return OPAL_SUCCESS;
+}
+
+static int
+mca_pml_cm_component_open(void)
+{
+    int ret;
+
+    ret = ompi_mtl_base_open();
+    if (OMPI_SUCCESS == ret) {
+      /* If no MTL components initialized CM component can be unloaded */
+      if (0 == opal_list_get_size(&ompi_mtl_base_components_opened)) {
+	ret = OPAL_ERR_NOT_AVAILABLE;
+      }
+    }
+    
+    return ret;
 }
 
 
@@ -140,12 +155,13 @@ mca_pml_cm_component_init(int* priority,
     if (OMPI_SUCCESS != ret) { 
         *priority = -1;
         return NULL;
-    } else if(strcmp(ompi_mtl_base_selected_component->mtl_version.mca_component_name, "psm") != 0) {
-        /* if mtl is not PSM then back down priority, and require the user to */
-        /*  specify pml cm directly if that is what they want priority */
-        /*  of 1 is sufficient in that case as it is the only pml that */ 
-        /*  will be considered */
-        *priority = 1;
+    } else if(strcmp(ompi_mtl_base_selected_component->mtl_version.mca_component_name, "psm") == 0) {
+        /*
+         * If MTL is PSM then up our priority
+         * For every other communication layer having MTLs and BTLs, the user/admin
+         * may still select PML/ob1 (BTLs) or PML/cm (MTLs) if preferable for the app/site.
+         */
+        *priority = 30;
     }
 
     
