@@ -58,9 +58,7 @@
 
 #include "opal_config.h"
 
-#include "opal/class/opal_value_array.h"
 #include "opal/class/opal_list.h"
-#include "opal/class/opal_hash_table.h"
 #include "opal/mca/mca.h"
 
 
@@ -145,9 +143,7 @@ typedef struct mca_base_param_info_t mca_base_param_info_t;
  * Global functions for MCA
  */
 
-#if defined(c_plusplus) || defined(__cplusplus)
-extern "C" {
-#endif
+BEGIN_C_DECLS
 
     /**
      * Make a real object for the info
@@ -325,6 +321,9 @@ extern "C" {
      * never be NULL. It will always have a value, even if that value is
      * the empty string.
      *
+     * Strings returned in the \em current_value parameter should later
+     * be free()'ed.
+     *
      * This function is identical to mca_base_param_reg_int() except
      * that you are registering a string parameter with an associated
      * string default value (which is \em not allowed to be NULL).
@@ -513,7 +512,7 @@ extern "C" {
      * never be NULL. It will always have a value, even if that value is
      * the empty string.
      * 
-     * Strings returns in the \em value parameter should later be
+     * Strings returned in the \em value parameter should later be
      * free()'ed.
      *
      * The value of a specific MCA parameter can be looked up using the
@@ -526,8 +525,10 @@ extern "C" {
      *
      * @param index [in] Index of MCA parameter to set
      * @param source [out] Enum value indicating source
-     * @param source_file [out] If value came from source, name of the file
-     * that set it.  The caller should not modify or free this string.
+     * @param source_file [out] If value came from source, name of the
+     * file that set it.  The caller should not modify or free this
+     * string.  It is permissable to specify source_file==NULL if the
+     * caller does not care to know the filename.
      *
      * @retval OPAL_ERROR If the parameter was not found.
      * @retval OPAL_SUCCESS Upon success.
@@ -535,9 +536,9 @@ extern "C" {
      * This function looks up to see where the value of an MCA
      * parameter came from.
      */
-    OPAL_DECLSPEC bool mca_base_param_lookup_source(int index, 
-                                                    mca_base_param_source_t *source,
-                                                    char **source_file);
+    OPAL_DECLSPEC int mca_base_param_lookup_source(int index, 
+                                                   mca_base_param_source_t *source,
+                                                   char **source_file);
 
     /**
      * Sets an "override" value for an integer MCA parameter.
@@ -548,7 +549,7 @@ extern "C" {
      * @retval OPAL_ERROR If the parameter was not found.
      * @retval OPAL_SUCCESS Upon success.
      *
-     * This function sets an integer value on the MCA parmeter
+     * This function sets an integer value on the MCA parameter
      * indicated by the index value index.  This value will be used in
      * lieu of any other value from any other MCA source (environment
      * variable, file, etc.) until the value is unset with
@@ -568,7 +569,7 @@ extern "C" {
      * @retval OPAL_ERROR If the parameter was not found.
      * @retval OPAL_SUCCESS Upon success.
      *
-     * This function sets a string value on the MCA parmeter
+     * This function sets a string value on the MCA parameter
      * indicated by the index value index.  This value will be used in
      * lieu of any other value from any other MCA source (environment
      * variable, file, etc.) until the value is unset with
@@ -855,8 +856,6 @@ extern "C" {
                                                         const char *comp,
                                                         const char *param) /* __opal_attribute_deprecated__ */;
 
-#if defined(c_plusplus) || defined(__cplusplus)
-}
-#endif
+END_C_DECLS
 
 #endif /* OPAL_MCA_BASE_PARAM_H */
