@@ -29,7 +29,6 @@
 #include <fcntl.h>
 
 #include "opal/util/output.h"
-#include "opal/util/show_help.h"
 #include "opal/util/argv.h"
 #include "opal/constants.h"
 
@@ -270,7 +269,10 @@ int opal_crs_blcr_module_finalize(void)
     return OPAL_SUCCESS;
 }
 
-int opal_crs_blcr_checkpoint(pid_t pid, opal_crs_base_snapshot_t *base_snapshot,  opal_crs_state_type_t *state)
+int opal_crs_blcr_checkpoint(pid_t pid,
+                             opal_crs_base_snapshot_t *base_snapshot,
+                             opal_crs_base_ckpt_options_t *options,
+                             opal_crs_state_type_t *state)
 {
     int ret, exit_status = OPAL_SUCCESS;
     opal_crs_blcr_snapshot_t *snapshot = OBJ_NEW(opal_crs_blcr_snapshot_t);
@@ -315,7 +317,11 @@ int opal_crs_blcr_checkpoint(pid_t pid, opal_crs_base_snapshot_t *base_snapshot,
         char *loc_fname = NULL;
 
         blcr_get_checkpoint_filename(&(snapshot->context_filename), pid);
-        asprintf(&loc_fname, "%s/%s", snapshot->super.local_location, snapshot->context_filename);
+        if( opal_crs_blcr_dev_null ) {
+            loc_fname = strdup("/dev/null");
+        } else {
+            asprintf(&loc_fname, "%s/%s", snapshot->super.local_location, snapshot->context_filename);
+        }
 
         opal_output_verbose(10, mca_crs_blcr_component.super.output_handle,
                             "crs:blcr: checkpoint SELF <%s>",
@@ -332,13 +338,16 @@ int opal_crs_blcr_checkpoint(pid_t pid, opal_crs_base_snapshot_t *base_snapshot,
                 opal_output(mca_crs_blcr_component.super.output_handle,
                             "crs:blcr: checkpoint(): Error: Unable to open checkpoint file (%s) for pid (%d)",
                             loc_fname, pid);
-                exit_status = ret;
+                exit_status = OPAL_ERROR;
                 goto cleanup;
             }
 
             cr_initialize_checkpoint_args_t(&cr_args);
             cr_args.cr_scope = CR_SCOPE_PROC;
             cr_args.cr_fd    = fd;
+            if( options->stop ) {
+                cr_args.cr_signal = SIGSTOP;
+            }
 
             ret = cr_request_checkpoint(&cr_args, &cr_handle);
             if( ret < 0 ) {
@@ -584,7 +593,7 @@ static int blcr_checkpoint_peer(pid_t pid, char * local_dir, char ** fname)
     int ret;
     pid_t child_pid;
     int exit_status = OPAL_SUCCESS;
-    int status, child_status;        
+    int status, child_status;
 
     opal_output_verbose(10, mca_crs_blcr_component.super.output_handle,
                         "crs:blcr: checkpoint_peer(%d, --)", pid);
