@@ -70,6 +70,7 @@
 #include <process.h>
 #include <signal.h>
 #include <conio.h>
+#include <fcntl.h>
 #ifdef HAVE_STDINT_H
 #include <stdint.h>
 #endif
@@ -90,6 +91,7 @@
 #include "opal/win32/ompi_utsname.h"
 #include "opal/win32/ompi_util.h"
 #include "opal/win32/ompi_misc.h"
+#include "opal/win32/ompi_inet.h"
 #endif
 
 #define MAXPATHLEN _MAX_PATH
@@ -149,7 +151,9 @@ typedef unsigned int uint;
 #define strncasecmp               _strnicmp
 #define strcasecmp                _stricmp
 #define umask                     _umask
-#define getch                     _getch 
+#define getch                     _getch
+#define random                    rand
+#define nanosleep(tp, rem)        Sleep(*tp.tv_sec*1000+*tp.tv_nsec/1000000)
 
 #ifndef UINT32_MAX
 #define UINT32_MAX            _UI32_MAX
@@ -199,7 +203,6 @@ typedef unsigned int uint;
  */
 #define SIGCHLD SIGILL
 #define SIGKILL WM_QUIT
-#define SIGPIPE 13
 #define SIGCONT 18
 #define SIGSTOP 19
 #define SIGTSTP 20
@@ -214,6 +217,29 @@ typedef unsigned int uint;
 
 #define sigset_t int
 #define in_addr_t uint32_t
+ 
+/* Need to define _Bool here for different version of VS.  
+   The definition in opal_config_bottom.h won't help,  
+   as long as we have a mixed C and C++ projects in one solution. */ 
+#if defined(_MSC_VER) && _MSC_VER < 1600 
+#define _Bool BOOL 
+#else 
+#define _Bool bool 
+#endif
+
+/*
+ * No syslog.h on Windows, but these have to be defined somehow.
+ * There could also be a notifier component later for Windows.
+ */
+
+#define LOG_EMERG   0
+#define LOG_ALERT   1
+#define LOG_CRIT    2
+#define LOG_ERR     3
+#define LOG_WARNING 4
+#define LOG_NOTICE  5
+#define LOG_INFO    6
+#define LOG_DEBUG   7
 
 /*
  * Mask these to Windows equivalents
