@@ -2,13 +2,14 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2009 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2011      Oracle and/or its affiliates.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -20,6 +21,9 @@
 #include "orte_config.h"
 #include "orte/constants.h"
 
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
@@ -32,20 +36,17 @@
 #include "opal/util/argv.h"
 #include "opal/util/path.h"
 #include "opal/mca/base/mca_base_param.h"
+#include "opal/mca/sysinfo/sysinfo.h"
 #include "opal/mca/installdirs/installdirs.h"
-#include "opal/class/opal_pointer_array.h"
-#include "opal/class/opal_value_array.h"
-#include "opal/hash_string.h"
+#include "opal/mca/paffinity/paffinity.h"
 
 #include "orte/util/show_help.h"
 #include "orte/util/proc_info.h"
 #include "orte/mca/errmgr/errmgr.h"
-#include "orte/mca/iof/iof.h"
-#include "orte/mca/rml/rml.h"
-#include "orte/mca/rml/base/rml_contact.h"
 #include "orte/mca/routed/routed.h"
 #include "orte/util/name_fns.h"
 #include "orte/runtime/orte_globals.h"
+#include "orte/util/nidmap.h"
 
 #include "orte/mca/ess/ess.h"
 #include "orte/mca/ess/base/base.h"
@@ -66,42 +67,33 @@ static void set_handler_default(int sig)
 #endif /* !defined(__WINDOWS__) */
 }
 
-static int rte_init(char flags);
+static int rte_init(void);
 static int rte_finalize(void);
-static bool proc_is_local(orte_process_name_t *proc);
+static uint8_t proc_get_locality(orte_process_name_t *proc);
+static orte_vpid_t proc_get_daemon(orte_process_name_t *proc);
 static char* proc_get_hostname(orte_process_name_t *proc);
-static uint32_t proc_get_arch(orte_process_name_t *proc);
 static orte_local_rank_t proc_get_local_rank(orte_process_name_t *proc);
 static orte_node_rank_t proc_get_node_rank(orte_process_name_t *proc);
-static int update_arch(orte_process_name_t *proc, uint32_t arch);
+static int update_pidmap(opal_byte_object_t *bo);
+static int update_nidmap(opal_byte_object_t *bo);
 
 orte_ess_base_module_t orte_ess_singleton_module = {
     rte_init,
     rte_finalize,
     orte_ess_base_app_abort,
-    proc_is_local,
+    proc_get_locality,
+    proc_get_daemon,
     proc_get_hostname,
-    proc_get_arch,
     proc_get_local_rank,
     proc_get_node_rank,
-    update_arch,
+    update_pidmap,
+    update_nidmap,
     NULL /* ft_event */
 };
 
-static opal_pointer_array_t nidmap;
-static opal_pointer_array_t jobmap;
-static orte_vpid_t nprocs;
-
-static int rte_init(char flags)
+static int rte_init(void)
 {
     int rc;
-    orte_nid_t *node;
-    orte_jmap_t *jmap;
-    orte_pmap_t pmap;
-    char *server_uri, *param;
-    uint16_t jobfam;
-    uint32_t hash32;
-    uint32_t bias;
     
     /* run the prolog */
     if (ORTE_SUCCESS != (rc = orte_ess_base_std_prolog())) {
@@ -109,127 +101,44 @@ static int rte_init(char flags)
         return rc;
     }
     
-    /* look for the ompi-server MCA param */
-    mca_base_param_reg_string_name("orte", "server",
-                                   "Server to be used as HNP - [file|FILE]:<filename> or just uri",
-                                   false, false, NULL, &server_uri);
-    
-    if (NULL != server_uri) {
-        /* we are going to connect to a server HNP */
-        if (0 == strncmp(server_uri, "file", strlen("file")) ||
-            0 == strncmp(server_uri, "FILE", strlen("FILE"))) {
-            char input[1024], *filename;
-            FILE *fp;
-            
-            /* it is a file - get the filename */
-            filename = strchr(server_uri, ':');
-            if (NULL == filename) {
-                /* filename is not correctly formatted */
-                orte_show_help("help-orterun.txt", "orterun:ompi-server-filename-bad", true,
-                               "singleton", server_uri);
-                return ORTE_ERROR;
-            }
-            ++filename; /* space past the : */
-            
-            if (0 >= strlen(filename)) {
-                /* they forgot to give us the name! */
-                orte_show_help("help-orterun.txt", "orterun:ompi-server-filename-missing", true,
-                               "singleton", server_uri);
-                return ORTE_ERROR;
-            }
-            
-            /* open the file and extract the uri */
-            fp = fopen(filename, "r");
-            if (NULL == fp) { /* can't find or read file! */
-                orte_show_help("help-orterun.txt", "orterun:ompi-server-filename-access", true,
-                               "singleton", server_uri);
-                return ORTE_ERROR;
-            }
-            if (NULL == fgets(input, 1024, fp)) {
-                /* something malformed about file */
-                fclose(fp);
-                orte_show_help("help-orterun.txt", "orterun:ompi-server-file-bad", true,
-                               "singleton", server_uri, "singleton");
-                return ORTE_ERROR;
-            }
-            fclose(fp);
-            input[strlen(input)-1] = '\0';  /* remove newline */
-            orte_process_info.my_hnp_uri = strdup(input);
-        } else {
-            orte_process_info.my_hnp_uri = strdup(server_uri);
-        }
-        /* save the daemon uri - we will process it later */
-        orte_process_info.my_daemon_uri = strdup(orte_process_info.my_hnp_uri);
-        /* indicate we are a singleton so orte_init knows what to do */
-        orte_process_info.singleton = true;
-        /* for convenience, push the pubsub version of this param into the environ */
-        asprintf(&param, "OMPI_MCA_dpm_orte_server=%s", orte_process_info.my_hnp_uri);
-        putenv(param);
-        free(param);
-        /* now define my own name */
-        /* hash the nodename */
-        OPAL_HASH_STR(orte_process_info.nodename, hash32);
-        
-        bias = (uint32_t)orte_process_info.pid;
-        
-        OPAL_OUTPUT_VERBOSE((5, orte_ess_base_output,
-                             "ess:singleton: initial bias %ld nodename hash %lu",
-                             (long)bias, (unsigned long)hash32));
-        
-        /* fold in the bias */
-        hash32 = hash32 ^ bias;
-        
-        /* now compress to 16-bits */
-        jobfam = (uint16_t)(((0x0000ffff & (0xffff0000 & hash32) >> 16)) ^ (0x0000ffff & hash32));
-        
-        OPAL_OUTPUT_VERBOSE((5, orte_ess_base_output,
-                             "ess:singleton:: final jobfam %lu",
-                             (unsigned long)jobfam));
-        
-        /* set the name */
-        ORTE_PROC_MY_NAME->jobid = 0xffff0000 & ((uint32_t)jobfam << 16);
-        ORTE_PROC_MY_NAME->vpid = 0;
-        
-    } else {
-        /*
-         * If we are the selected module, then we must be a singleton
-         * as it means that no other method for discovering a name
-         * could be found. In this case, we need to start a daemon that
-         * can support our operation. We must do this for two reasons:
-         *
-         * (1) if we try to play the role of the HNP, then any child processes
-         * we might start via comm_spawn will rely on us for all ORTE-level
-         * support. However, we can only progress those requests when the
-         * the application calls into the OMPI/ORTE library! Thus, if this
-         * singleton just does computation, the other processes will "hang"
-         * in any calls into the ORTE layer that communicate with the HNP -
-         * and most calls on application procs *do*.
-         *
-         * (2) daemons are used to communicate messages for administrative
-         * purposes in a broadcast-like manner. Thus, daemons are expected
-         * to be able to interpret specific commands. Our application process
-         * doesn't have any idea how to handle those commands, thus causing
-         * the entire ORTE administrative system to break down.
-         *
-         * For those reasons, we choose to fork/exec a daemon at this time
-         * and then reconnect ourselves to it. We could just "fork" and declare
-         * the child to be a daemon, but that would require we place *all* of the
-         * daemon command processing code in the ORTE library, do some strange
-         * mojo in a few places, etc. This doesn't seem worth it, so we'll just
-         * do the old fork/exec here
-         *
-         * Note that Windows-based systems have to do their own special trick as
-         * they don't support fork/exec. So we have to use a giant "if" here to
-         * protect the Windows world. To make the results more readable, we put
-         * the whole mess in a separate function below
-         */
-        if (ORTE_SUCCESS != (rc= fork_hnp())) {
-            /* if this didn't work, then we cannot support operation any further.
-             * Abort the system and tell orte_init to exit
-             */
-            ORTE_ERROR_LOG(rc);
-            return rc;
-        }
+    /*
+     * If we are the selected module, then we must be a singleton
+     * as it means that no other method for discovering a name
+     * could be found. In this case, we need to start a daemon that
+     * can support our operation. We must do this for two reasons:
+     *
+     * (1) if we try to play the role of the HNP, then any child processes
+     * we might start via comm_spawn will rely on us for all ORTE-level
+     * support. However, we can only progress those requests when the
+     * the application calls into the OMPI/ORTE library! Thus, if this
+     * singleton just does computation, the other processes will "hang"
+     * in any calls into the ORTE layer that communicate with the HNP -
+     * and most calls on application procs *do*.
+     *
+     * (2) daemons are used to communicate messages for administrative
+     * purposes in a broadcast-like manner. Thus, daemons are expected
+     * to be able to interpret specific commands. Our application process
+     * doesn't have any idea how to handle those commands, thus causing
+     * the entire ORTE administrative system to break down.
+     *
+     * For those reasons, we choose to fork/exec a daemon at this time
+     * and then reconnect ourselves to it. We could just "fork" and declare
+     * the child to be a daemon, but that would require we place *all* of the
+     * daemon command processing code in the ORTE library, do some strange
+     * mojo in a few places, etc. This doesn't seem worth it, so we'll just
+     * do the old fork/exec here
+     *
+     * Note that Windows-based systems have to do their own special trick as
+     * they don't support fork/exec. So we have to use a giant "if" here to
+     * protect the Windows world. To make the results more readable, we put
+     * the whole mess in a separate function below
+     */
+    if (ORTE_SUCCESS != (rc= fork_hnp())) {
+        /* if this didn't work, then we cannot support operation any further.
+        * Abort the system and tell orte_init to exit
+        */
+        ORTE_ERROR_LOG(rc);
+        return rc;
     }
 
     orte_process_info.num_procs = 1;
@@ -239,39 +148,16 @@ static int rte_init(char flags)
      * library wrt pty's and stdin
      */
 
-    /* setup the nidmap arrays */
-    OBJ_CONSTRUCT(&nidmap, opal_pointer_array_t);
-    opal_pointer_array_init(&nidmap, 1,
-                            INT32_MAX, 8);
-    
-    /* setup array of jmaps */
-    OBJ_CONSTRUCT(&jobmap, opal_pointer_array_t);
-    opal_pointer_array_init(&jobmap, 1, INT32_MAX, 1);
-    jmap = OBJ_NEW(orte_jmap_t);
-    jmap->job = ORTE_PROC_MY_NAME->jobid;
-    opal_pointer_array_add(&jobmap, jmap);
-    
-    /* we cannot use the std nidmap construction in the ess/base because
-     * the daemon couldn't pass us the info! Since we are a singleton, we
-     * already -know- the info, so we will construct it ourselves
-     */
-    
-    /* create a nidmap entry for this node */
-    node = OBJ_NEW(orte_nid_t);
-    node->name = strdup(orte_process_info.nodename);
-    node->daemon = 0;  /* the HNP co-occupies our node */
-    node->arch = orte_process_info.arch;
-    opal_pointer_array_set_item(&nidmap, 0, node);
-    nprocs = 1;
-    
-    /* likewise, we need to construct our own jobmap. Again, since we are
-     * a singleton, this is rather trivial
-     */
-    pmap.local_rank = 0;
-    pmap.node_rank = 0;
-    pmap.node = 0;
-    opal_value_array_set_item(&jmap->pmap, 0, &pmap);
-    
+    /* setup the nidmap and jobmap arrays */
+    if (ORTE_SUCCESS != (rc = orte_util_nidmap_init(NULL))) {
+        ORTE_ERROR_LOG(rc);
+        return rc;
+    }
+    if (ORTE_SUCCESS != (rc = orte_util_setup_local_nidmap_entries())) {
+        ORTE_ERROR_LOG(rc);
+        return rc;
+    }
+        
     /* use the std app init to complete the procedure */
     if (ORTE_SUCCESS != (rc = orte_ess_base_app_setup())) {
         ORTE_ERROR_LOG(rc);
@@ -284,21 +170,9 @@ static int rte_init(char flags)
 static int rte_finalize(void)
 {
     int ret;
-    orte_nid_t **nids;
-    orte_jmap_t **jmaps;
-    int32_t i;
     
     /* deconstruct my nidmap and jobmap arrays */
-    nids = (orte_nid_t**)nidmap.addr;
-    for (i=0; i < nidmap.size && NULL != nids[i]; i++) {
-        OBJ_RELEASE(nids[i]);
-    }
-    OBJ_DESTRUCT(&nidmap);
-    jmaps = (orte_jmap_t**)jobmap.addr;
-    for (i=0; i < jobmap.size && NULL != jmaps[i]; i++) {
-        OBJ_RELEASE(jmaps[i]);
-    }
-    OBJ_DESTRUCT(&jobmap);
+    orte_util_nidmap_finalize();
     
     /* use the default procedure to finish */
     if (ORTE_SUCCESS != (ret = orte_ess_base_app_finalize())) {
@@ -325,7 +199,7 @@ static int fork_hnp(void)
     int rc;
     
     /* A pipe is used to communicate between the parent and child to
-        indicate whether the exec ultiimately succeeded or failed.  The
+        indicate whether the exec ultimately succeeded or failed.  The
         child sets the pipe to be close-on-exec; the child only ever
         writes anything to the pipe if there is an error (e.g.,
         executable not found, exec() fails, etc.).  The parent does a
@@ -471,7 +345,7 @@ static int fork_hnp(void)
             return ORTE_ERR_HNP_COULD_NOT_START;
         }
         
-        /* parse the name from the returned info */
+        /* parse the sysinfo from the returned info */
         if (']' != orted_uri[strlen(orted_uri)-1]) {
             ORTE_ERROR_LOG(ORTE_ERR_COMM_FAILURE);
             free(orted_uri);
@@ -483,6 +357,23 @@ static int fork_hnp(void)
             free(orted_uri);
             return ORTE_ERR_COMM_FAILURE;
         }
+        param[-1] = '\0';  /* terminate the string */
+	
+	if (ORTE_SUCCESS !=
+	    (rc = orte_util_convert_string_to_sysinfo(&orte_local_cpu_type,
+						      &orte_local_cpu_model,
+						      ++param))) {
+		ORTE_ERROR_LOG(rc);
+		free(orted_uri);
+		return rc;
+	}
+
+        /* parse the name from the returned info */
+        if (NULL == (param = strrchr(orted_uri, '['))) {
+            ORTE_ERROR_LOG(ORTE_ERR_COMM_FAILURE);
+            free(orted_uri);
+            return ORTE_ERR_COMM_FAILURE;
+        }
         *param = '\0';  /* terminate the string */
         param++;
         if (ORTE_SUCCESS != (rc = orte_util_convert_string_to_process_name(ORTE_PROC_MY_NAME, param))) {
@@ -497,25 +388,149 @@ static int fork_hnp(void)
         orte_process_info.my_hnp_uri = strdup(orted_uri);
         
        /* indicate we are a singleton so orte_init knows what to do */
-        orte_process_info.singleton = true;
+        orte_process_info.proc_type |= ORTE_PROC_SINGLETON;
         /* all done - report success */
         free(orted_uri);
         return ORTE_SUCCESS;
     }
 #else
-    /* someone will have to devise a Windows equivalent */
-#endif    
-    
+    int p[2], death_pipe[2];
+    char *cmd;
+    char **argv = NULL;
+    int argc;
+    char *param;
+    int buffer_length, num_chars_read, chunk;
+    char *orted_uri;
+    int rc;
+
+    /* Use socket to communicate between the parent and child to
+       indicate whether the spawn  succeeded or failed.*/
+    if (evutil_socketpair(AF_UNIX, SOCK_STREAM, 0, p) == -1) {
+        return ORTE_ERROR;
+    }
+
+    /* Set another pair socket to watch if we terminated. */
+    if (evutil_socketpair(AF_UNIX, SOCK_STREAM, 0, death_pipe) == -1) {
+        return ORTE_ERROR;
+    }
+
+    /* find the orted binary using the install_dirs support - this also
+     * checks to ensure that we can see this executable and it *is* executable by us
+     */
+    cmd = opal_path_access("orted.exe", opal_install_dirs.bindir, X_OK);
+    if (NULL == cmd) {
+        /* guess we couldn't do it - best to abort */
+        ORTE_ERROR_LOG(ORTE_ERR_FILE_NOT_EXECUTABLE);
+
+        closesocket(p[0]);
+        closesocket(p[1]);
+        return ORTE_ERR_FILE_NOT_EXECUTABLE;
+    }
+
+    /* okay, setup an appropriate argv */
+    opal_argv_append(&argc, &argv, "orted.exe");
+
+    /* tell the daemon it is to be the HNP */
+    opal_argv_append(&argc, &argv, "--hnp");
+
+    /* tell the daemon to get out of our process group */
+    opal_argv_append(&argc, &argv, "--set-sid");
+
+    /* tell the daemon to report back its uri so we can connect to it */
+    opal_argv_append(&argc, &argv, "--report-uri");
+    asprintf(&param, "%d", p[1]);
+    opal_argv_append(&argc, &argv, param);
+    free(param);
+
+    /* give the daemon a socket number it can watch to tell when we have died */
+    opal_argv_append(&argc, &argv, "--singleton-died-pipe");
+    asprintf(&param, "%d", death_pipe[0]);
+    opal_argv_append(&argc, &argv, param);
+    free(param);
+
+    /* add any debug flags */
+    if (orte_debug_flag) {
+        opal_argv_append(&argc, &argv, "--debug");
+    }
+
+    if (orte_debug_daemons_flag) {
+        opal_argv_append(&argc, &argv, "--debug-daemons");
+    }
+
+    if (orte_debug_daemons_file_flag) {
+        if (!orte_debug_daemons_flag) {
+            opal_argv_append(&argc, &argv, "--debug-daemons");
+        }
+        opal_argv_append(&argc, &argv, "--debug-daemons-file");
+    }
+
+    /* spawn the daemon. */
+    orte_process_info.hnp_pid = (int) _spawnvp( _P_NOWAIT, cmd, argv );
+
+    closesocket(p[1]);  /* parent closes the write - orted will write its contact info to it*/
+    closesocket(death_pipe[0]);  /* parent closes the death_pipe's read */
+
+    /* setup the buffer to read the name + uri */
+    buffer_length = ORTE_URI_MSG_LGTH;
+    chunk = ORTE_URI_MSG_LGTH-1;
+    num_chars_read = 0;
+    orted_uri = (char*)malloc(buffer_length);
+
+    while (chunk == (rc = recv(p[0], &orted_uri[num_chars_read], chunk, 0))) {
+        /* we read an entire buffer - better get more */
+        num_chars_read += chunk;
+        buffer_length += ORTE_URI_MSG_LGTH;
+        orted_uri = (char *) realloc((void*)orted_uri, buffer_length);
+    }
+    num_chars_read += rc;
+
+    if (num_chars_read <= 0) {
+        /* we didn't get anything back - this is bad */
+        ORTE_ERROR_LOG(ORTE_ERR_HNP_COULD_NOT_START);
+        free(orted_uri);
+        return ORTE_ERR_HNP_COULD_NOT_START;
+    }
+
+    /* parse the name from the returned info */
+    if (']' != orted_uri[strlen(orted_uri)-1]) {
+        ORTE_ERROR_LOG(ORTE_ERR_COMM_FAILURE);
+        free(orted_uri);
+        return ORTE_ERR_COMM_FAILURE;
+    }
+    orted_uri[strlen(orted_uri)-1] = '\0';
+    if (NULL == (param = strrchr(orted_uri, '['))) {
+        ORTE_ERROR_LOG(ORTE_ERR_COMM_FAILURE);
+        free(orted_uri);
+        return ORTE_ERR_COMM_FAILURE;
+    }
+    *param = '\0';  /* terminate the string */
+    param++;
+    if (ORTE_SUCCESS != (rc = orte_util_convert_string_to_process_name(ORTE_PROC_MY_NAME, param))) {
+        ORTE_ERROR_LOG(rc);
+        free(orted_uri);
+        return rc;
+    }
+    /* save the daemon uri - we will process it later */
+    orte_process_info.my_daemon_uri = strdup(orted_uri);
+
+    /* likewise, since this is also the HNP, set that uri too */
+    orte_process_info.my_hnp_uri = strdup(orted_uri);
+
+   /* indicate we are a singleton so orte_init knows what to do */
+    orte_process_info.proc_type |= ORTE_PROC_SINGLETON;
+    /* all done - report success */
+    free(orted_uri);
     return ORTE_SUCCESS;
+#endif
 }
 
-static bool proc_is_local(orte_process_name_t *proc)
+static uint8_t proc_get_locality(orte_process_name_t *proc)
 {
     orte_nid_t *nid;
     
-    if (NULL == (nid = orte_ess_base_lookup_nid(&nidmap, &jobmap, proc))) {
+    if (NULL == (nid = orte_util_lookup_nid(proc))) {
         ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
-        return false;
+        return OPAL_PROC_NON_LOCAL;
     }
     
     if (nid->daemon == ORTE_PROC_MY_DAEMON->vpid) {
@@ -523,7 +538,7 @@ static bool proc_is_local(orte_process_name_t *proc)
                              "%s ess:singleton: proc %s is LOCAL",
                              ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                              ORTE_NAME_PRINT(proc)));
-        return true;
+        return (OPAL_PROC_ON_NODE | OPAL_PROC_ON_CU | OPAL_PROC_ON_CLUSTER);
     }
     
     OPAL_OUTPUT_VERBOSE((2, orte_ess_base_output,
@@ -531,15 +546,36 @@ static bool proc_is_local(orte_process_name_t *proc)
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                          ORTE_NAME_PRINT(proc)));
     
-    return false;
+    return OPAL_PROC_NON_LOCAL;
     
 }
 
+static orte_vpid_t proc_get_daemon(orte_process_name_t *proc)
+{
+    orte_nid_t *nid;
+    
+    if( ORTE_JOBID_IS_DAEMON(proc->jobid) ) {
+        return proc->vpid;
+    }
+
+    if (NULL == (nid = orte_util_lookup_nid(proc))) {
+        return ORTE_VPID_INVALID;
+    }
+    
+    OPAL_OUTPUT_VERBOSE((2, orte_ess_base_output,
+                         "%s ess:singleton: proc %s is hosted by daemon %s",
+                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                         ORTE_NAME_PRINT(proc),
+                         ORTE_VPID_PRINT(nid->daemon)));
+    
+    return nid->daemon;
+}
+
 static char* proc_get_hostname(orte_process_name_t *proc)
 {
     orte_nid_t *nid;
     
-    if (NULL == (nid = orte_ess_base_lookup_nid(&nidmap, &jobmap, proc))) {
+    if (NULL == (nid = orte_util_lookup_nid(proc))) {
         ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
         return NULL;
     }
@@ -553,49 +589,11 @@ static char* proc_get_hostname(orte_process_name_t *proc)
     return nid->name;
 }
 
-static uint32_t proc_get_arch(orte_process_name_t *proc)
-{
-    orte_nid_t *nid;
-    
-    if (NULL == (nid = orte_ess_base_lookup_nid(&nidmap, &jobmap, proc))) {
-        ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
-        return 0;
-    }
-    
-    OPAL_OUTPUT_VERBOSE((2, orte_ess_base_output,
-                         "%s ess:singleton: proc %s has arch %0x",
-                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                         ORTE_NAME_PRINT(proc),
-                         nid->arch));
-    
-    return nid->arch;
-}
-
-static int update_arch(orte_process_name_t *proc, uint32_t arch)
-{
-    orte_nid_t *nid;
-    
-    if (NULL == (nid = orte_ess_base_lookup_nid(&nidmap, &jobmap, proc))) {
-        ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
-        return ORTE_ERR_NOT_FOUND;
-    }
-    
-    OPAL_OUTPUT_VERBOSE((2, orte_ess_base_output,
-                         "%s ess:singleton: updating proc %s to arch %0x",
-                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                         ORTE_NAME_PRINT(proc),
-                         arch));
-    
-    nid->arch = arch;
-    
-    return ORTE_SUCCESS;
-}
-
 static orte_local_rank_t proc_get_local_rank(orte_process_name_t *proc)
 {
     orte_pmap_t *pmap;
     
-    if (NULL == (pmap = orte_ess_base_lookup_pmap(&jobmap, proc))) {
+    if (NULL == (pmap = orte_util_lookup_pmap(proc))) {
         ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
         return ORTE_LOCAL_RANK_INVALID;
     }    
@@ -613,7 +611,7 @@ static orte_node_rank_t proc_get_node_rank(orte_process_name_t *proc)
 {
     orte_pmap_t *pmap;
     
-    if (NULL == (pmap = orte_ess_base_lookup_pmap(&jobmap, proc))) {
+    if (NULL == (pmap = orte_util_lookup_pmap(proc))) {
         ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
         return ORTE_NODE_RANK_INVALID;
     }    
@@ -626,3 +624,26 @@ static orte_node_rank_t proc_get_node_rank(orte_process_name_t *proc)
     
     return pmap->node_rank;
 }
+
+static int update_pidmap(opal_byte_object_t *bo)
+{
+    int ret;
+    
+    /* build the pmap */
+    if (ORTE_SUCCESS != (ret = orte_util_decode_pidmap(bo))) {
+        ORTE_ERROR_LOG(ret);
+    }
+    
+    return ret;
+}
+
+static int update_nidmap(opal_byte_object_t *bo)
+{
+    int rc;
+    /* decode the nidmap - the util will know what to do */
+    if (ORTE_SUCCESS != (rc = orte_util_decode_nodemap(bo))) {
+        ORTE_ERROR_LOG(rc);
+    }    
+    return rc;
+}
+
