@@ -26,9 +26,10 @@
  */
 
 #include "orte_config.h"
-#include "orte/constants.h"
-#include "orte/types.h"
 
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
 #include <sys/types.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -52,19 +53,24 @@
 
 #include "opal/mca/installdirs/installdirs.h"
 #include "opal/util/argv.h"
+#include "opal/util/output.h"
 #include "opal/util/opal_environ.h"
 #include "opal/util/path.h"
 #include "opal/util/basename.h"
 #include "opal/mca/base/mca_base_param.h"
 
+#include "orte/constants.h"
+#include "orte/types.h"
 #include "orte/util/show_help.h"
 #include "orte/util/name_fns.h"
+#include "orte/util/regex.h"
 #include "orte/runtime/orte_globals.h"
-#include "orte/runtime/runtime.h"
 #include "orte/runtime/orte_wait.h"
 #include "orte/mca/errmgr/errmgr.h"
 #include "orte/mca/rmaps/rmaps.h"
 
+#include "orte/orted/orted.h"
+
 #include "orte/mca/plm/plm.h"
 #include "orte/mca/plm/base/plm_private.h"
 #include "plm_slurm.h"
@@ -75,7 +81,6 @@
  */
 static int plm_slurm_init(void);
 static int plm_slurm_launch_job(orte_job_t *jdata);
-static int plm_slurm_terminate_job(orte_jobid_t jobid);
 static int plm_slurm_terminate_orteds(void);
 static int plm_slurm_signal_job(orte_jobid_t jobid, int32_t signal);
 static int plm_slurm_finalize(void);
@@ -92,8 +97,9 @@ orte_plm_base_module_1_0_0_t orte_plm_slurm_module = {
     orte_plm_base_set_hnp_name,
     plm_slurm_launch_job,
     NULL,
-    plm_slurm_terminate_job,
+    orte_plm_base_orted_terminate_job,
     plm_slurm_terminate_orteds,
+    orte_plm_base_orted_kill_local_procs,
     plm_slurm_signal_job,
     plm_slurm_finalize
 };
@@ -105,7 +111,7 @@ static pid_t primary_srun_pid = 0;
 static bool primary_pid_set = false;
 static orte_jobid_t active_job = ORTE_JOBID_INVALID;
 static bool launching_daemons;
-
+static bool local_launch_available = false;
 
 /**
 * Init the module
@@ -117,6 +123,14 @@ static int plm_slurm_init(void)
     if (ORTE_SUCCESS != (rc = orte_plm_base_comm_start())) {
         ORTE_ERROR_LOG(rc);
     }
+    
+    if (ORTE_SUCCESS == orte_plm_base_rsh_launch_agent_setup(NULL, NULL)) {
+        local_launch_available = true;
+    }
+    
+    /* we don't need a barrier to exit */
+    orte_orted_exit_with_barrier = false;
+
     return rc;
 }
 
@@ -140,7 +154,6 @@ static int plm_slurm_launch_job(orte_job_t *jdata)
     char* var;
     char *nodelist_flat;
     char **nodelist_argv;
-    int nodelist_argc;
     char *name_string;
     char **custom_strings;
     int num_args, i;
@@ -149,7 +162,35 @@ static int plm_slurm_launch_job(orte_job_t *jdata)
     int proc_vpid_index;
     orte_jobid_t failed_job;
     bool failed_launch=true;
+    bool using_regexp=false;
 
+    if (NULL == jdata) {
+	/* just launching debugger daemons */
+	active_job = ORTE_JOBID_INVALID;
+	goto launch_apps;
+    }
+
+    if (jdata->controls & ORTE_JOB_CONTROL_LOCAL_SLAVE) {
+        /* if this is a request to launch a local slave,
+         * then we will not be launching an orted - we will
+         * directly ssh the slave process itself. No mapping
+         * is performed to support this - the caller must
+         * provide all the info required to launch the job,
+         * including the target hosts
+         */
+        if (!local_launch_available) {
+            /* if we can't support this, then abort */
+            orte_show_help("help-plm-slurm.txt", "no-local-slave-support", true);
+            return ORTE_ERR_FAILED_TO_START;
+        }
+        return orte_plm_base_local_slave_launch(jdata);
+    }
+    
+    /* if we are timing, record the start time */
+    if (orte_timing) {
+        gettimeofday(&orte_plm_globals.daemonlaunchstart, NULL);
+    }
+    
     /* flag the daemons as failing by default */
     failed_job = ORTE_PROC_MY_NAME->jobid;
     
@@ -164,24 +205,18 @@ static int plm_slurm_launch_job(orte_job_t *jdata)
     /* indicate the state of the launch */
     launching_daemons = true;
     
-    /* create a jobid for this job */
-    if (ORTE_SUCCESS != (rc = orte_plm_base_create_jobid(&jdata->jobid))) {
-        ORTE_ERROR_LOG(rc);
-        goto cleanup;
-    }
-        
-    OPAL_OUTPUT_VERBOSE((1, orte_plm_globals.output,
-                         "%s plm:slurm: launching job %s",
-                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                         ORTE_JOBID_PRINT(jdata->jobid)));
-    
     /* setup the job */
     if (ORTE_SUCCESS != (rc = orte_plm_base_setup_job(jdata))) {
         ORTE_ERROR_LOG(rc);
         goto cleanup;
     }
 
-     /* set the active jobid */
+    OPAL_OUTPUT_VERBOSE((1, orte_plm_globals.output,
+                         "%s plm:slurm: launching job %s",
+                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                         ORTE_JOBID_PRINT(jdata->jobid)));
+    
+    /* set the active jobid */
      active_job = jdata->jobid;
     
     /* Get the map for this job */
@@ -240,7 +275,6 @@ static int plm_slurm_launch_job(orte_job_t *jdata)
 
     /* create nodelist */
     nodelist_argv = NULL;
-    nodelist_argc = 0;
 
     for (n=0; n < map->num_nodes; n++ ) {
         /* if the daemon already exists on this node, then
@@ -253,7 +287,7 @@ static int plm_slurm_launch_job(orte_job_t *jdata)
         /* otherwise, add it to the list of nodes upon which
          * we need to launch a daemon
          */
-        opal_argv_append(&nodelist_argc, &nodelist_argv, nodes[n]->name);
+        opal_argv_append_nosize(&nodelist_argv, nodes[n]->name);
     }
     if (0 == opal_argv_count(nodelist_argv)) {
         orte_show_help("help-plm-slurm.txt", "no-hosts-in-list", true);
@@ -279,9 +313,9 @@ static int plm_slurm_launch_job(orte_job_t *jdata)
     
    /* Add basic orted command line options, including debug flags */
     orte_plm_base_orted_append_basic_args(&argc, &argv,
-                                          "slurm", 
-                                          &proc_vpid_index,
-                                          false);
+                                          "slurm", &proc_vpid_index,
+                                          false, nodelist_flat);
+    free(nodelist_flat);
 
     /* tell the new daemons the base of the name list so they can compute
      * their own name on the other end
@@ -296,15 +330,6 @@ static int plm_slurm_launch_job(orte_job_t *jdata)
     argv[proc_vpid_index] = strdup(name_string);
     free(name_string);
 
-    if (0 < opal_output_get_verbosity(orte_plm_globals.output)) {
-        param = opal_argv_join(argv, ' ');
-        OPAL_OUTPUT_VERBOSE((1, orte_plm_globals.output,
-                             "%s plm:slurm: final top-level argv:\n\t%s",
-                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                             (NULL == param) ? "NULL" : param));
-        if (NULL != param) free(param);
-    }
-
     /* Copy the prefix-directory specified in the
        corresponding app_context.  If there are multiple,
        different prefix's in the app context, complain (i.e., only
@@ -325,7 +350,8 @@ static int plm_slurm_launch_job(orte_job_t *jdata)
             }
 
             /* If not yet set, copy it; iff set, then it's the
-               same anyway */
+             * same anyway
+             */
             if (NULL == cur_prefix) {
                 cur_prefix = strdup(app_prefix_dir);
                 OPAL_OUTPUT_VERBOSE((1, orte_plm_globals.output,
@@ -339,12 +365,34 @@ static int plm_slurm_launch_job(orte_job_t *jdata)
     /* setup environment */
     env = opal_argv_copy(orte_launch_environ);
 
-    /* add the nodelist */
-    var = mca_base_param_environ_variable("orte", "slurm", "nodelist");
-    opal_setenv(var, nodelist_flat, true, &env);
-    free(nodelist_flat);
+    /* enable local launch by the orteds */
+    var = mca_base_param_environ_variable("plm", NULL, NULL);
+    opal_setenv(var, "rsh", true, &env);
     free(var);
-
+    
+    /* if we can do it, use the regexp to launch the apps - this
+     * requires that the user requested this mode, that we were
+     * provided with static ports, and that we only have one
+     * app_context
+     */
+    if (orte_use_regexp && orte_static_ports && jdata->num_apps < 2) {
+        char *regexp;
+        regexp = orte_regex_encode_maps(jdata);
+        opal_argv_append(&argc, &argv, "--launch");
+        opal_argv_append(&argc, &argv, regexp);
+        free(regexp);
+        using_regexp = true;
+    }
+    
+    if (0 < opal_output_get_verbosity(orte_plm_globals.output)) {
+        param = opal_argv_join(argv, ' ');
+        OPAL_OUTPUT_VERBOSE((1, orte_plm_globals.output,
+                             "%s plm:slurm: final top-level argv:\n\t%s",
+                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                             (NULL == param) ? "NULL" : param));
+        if (NULL != param) free(param);
+    }
+    
     /* exec the daemon(s) */
     if (ORTE_SUCCESS != (rc = plm_slurm_start_proc(argc, argv, env, cur_prefix))) {
         ORTE_ERROR_LOG(rc);
@@ -369,12 +417,36 @@ launch_apps:
     /* get here if daemons launch okay - any failures now by apps */
     launching_daemons = false;
     failed_job = active_job;
-    if (ORTE_SUCCESS != (rc = orte_plm_base_launch_apps(active_job))) {
-        OPAL_OUTPUT_VERBOSE((1, orte_plm_globals.output,
-                             "%s plm:slurm: launch of apps failed for job %s on error %s",
-                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                             ORTE_JOBID_PRINT(active_job), ORTE_ERROR_NAME(rc)));
-        goto cleanup;
+    if (using_regexp) {
+        /* daemons already have launch cmd - just wait for them to
+         * report back
+         */
+        opal_buffer_t launch;
+        int8_t flag;
+        orte_daemon_cmd_flag_t command = ORTE_DAEMON_ADD_LOCAL_PROCS;
+        OBJ_CONSTRUCT(&launch, opal_buffer_t);
+        opal_dss.pack(&launch, &command, 1, ORTE_DAEMON_CMD);
+        flag = 1;
+        opal_dss.pack(&launch, &flag, 1, OPAL_INT8);
+        opal_dss.pack(&launch, &orted_launch_cmd, 1, OPAL_STRING);
+        ORTE_MESSAGE_EVENT(ORTE_PROC_MY_NAME, &launch, ORTE_RML_TAG_DAEMON, orte_daemon_cmd_processor);
+        OBJ_DESTRUCT(&launch);
+
+        if (ORTE_SUCCESS != (rc = orte_plm_base_report_launched(jdata->jobid))) {
+            OPAL_OUTPUT_VERBOSE((5, orte_plm_globals.output,
+                                 "%s plm:slurm:launch failed for job %s on error %s",
+                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                                 ORTE_JOBID_PRINT(jdata->jobid), ORTE_ERROR_NAME(rc)));
+            goto cleanup;
+        }
+    } else {
+        if (ORTE_SUCCESS != (rc = orte_plm_base_launch_apps(active_job))) {
+            OPAL_OUTPUT_VERBOSE((1, orte_plm_globals.output,
+                                 "%s plm:slurm: launch of apps failed for job %s on error %s",
+                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                                 ORTE_JOBID_PRINT(active_job), ORTE_ERROR_NAME(rc)));
+            goto cleanup;
+        }
     }
 
     /* declare the launch a success */
@@ -416,19 +488,6 @@ cleanup:
 }
 
 
-static int plm_slurm_terminate_job(orte_jobid_t jobid)
-{
-    int rc;
-    
-    /* order them to kill their local procs for this job */
-    if (ORTE_SUCCESS != (rc = orte_plm_base_orted_kill_local_procs(jobid))) {
-        ORTE_ERROR_LOG(rc);
-    }
-    
-    return rc;
-}
-
-
 /**
 * Terminate the orteds for a given job
  */
@@ -440,7 +499,7 @@ static int plm_slurm_terminate_orteds(void)
     /* tell them to die without sending a reply - we will rely on the
      * waitpid to tell us when they have exited!
      */
-    if (ORTE_SUCCESS != (rc = orte_plm_base_orted_exit(ORTE_DAEMON_EXIT_NO_REPLY_CMD))) {
+    if (ORTE_SUCCESS != (rc = orte_plm_base_orted_exit(ORTE_DAEMON_EXIT_CMD))) {
         ORTE_ERROR_LOG(rc);
     }
     
@@ -493,8 +552,7 @@ static int plm_slurm_finalize(void)
 }
 
 
-static void srun_wait_cb(pid_t pid, int status, void* cbdata)
-{
+static void srun_wait_cb(pid_t pid, int status, void* cbdata){
     orte_job_t *jdata;
     
     /* According to the SLURM folks, srun always returns the highest exit
@@ -667,7 +725,7 @@ static int plm_slurm_start_proc(int argc, char **argv, char **env,
             primary_srun_pid = srun_pid;
             primary_pid_set = true;
         }
-
+        
         /* setup the waitpid so we can find out if srun succeeds! */
         orte_wait_cb(srun_pid, srun_wait_cb, NULL);
         free(exec_argv);
